import Oasys.gRPC


# Metaclass for static properties and constants
class ColourType(type):
    _consts = {'BACKGROUND', 'BLACK', 'BLUE', 'CYAN', 'DARK_GREEN', 'DARK_GREY', 'DARK_MAGENTA', 'FOREGROUND', 'GOLD', 'GREEN', 'HOT_PINK', 'INDIGO', 'LIGHT_GREY', 'LIGHT_PINK', 'LIME', 'MAGENTA', 'MAROON', 'MEDIUM_BLUE', 'MEDIUM_GREEN', 'MEDIUM_GREY', 'NAVY', 'OLIVE', 'ORANGE', 'PALE_YELLOW', 'PINK', 'PURPLE', 'RED', 'SEA_GREEN', 'SKY', 'TURQUOISE', 'USER_1', 'USER_2', 'USER_3', 'USER_4', 'USER_5', 'USER_6', 'USER_n (n = 1 to 150)', 'WHITE', 'YELLOW'}

    def __getattr__(cls, name):
        if name in ColourType._consts:
            return Oasys.THIS._connection.classGetter(cls.__name__, name)

        raise AttributeError


class Colour(Oasys.gRPC.OasysItem, metaclass=ColourType):


    def __del__(self):
        if not Oasys.THIS._connection:
            return

        Oasys.THIS._connection.destructor(self.__class__.__name__, self._handle)


    def __getattr__(self, name):
        raise AttributeError


    def __setattr__(self, name, value):
# Set the property locally
        self.__dict__[name] = value


# Static methods
    def GetFromName(name):
        """
        Returns the colour for a given core or user colour name

        Parameters
        ----------
        name : string
            The name of the colour, for example red or user_green or green/cyan

        Returns
        -------
        int
            colour value (integer)
        """
        return Oasys.THIS._connection.classMethod(__class__.__name__, "GetFromName", name)

    def RGB(red, green, blue):
        """
        Creates a colour from red, green and blue components

        Parameters
        ----------
        red : integer
            red component of colour (0-255)
        green : integer
            green component of colour (0-255)
        blue : integer
            blue component of colour (0-255)

        Returns
        -------
        int
            colour value (integer)
        """
        return Oasys.THIS._connection.classMethod(__class__.__name__, "RGB", red, green, blue)




import Oasys.gRPC


# Metaclass for static properties and constants
class ModelType(type):
    _consts = {'ALL_FILES', 'ASCII', 'LSDA', 'THF', 'XTF', 'ZTF'}

    def __getattr__(cls, name):
        if name in ModelType._consts:
            return Oasys.THIS._connection.classGetter(cls.__name__, name)

        raise AttributeError


class Model(Oasys.gRPC.OasysItem, metaclass=ModelType):
    _props = {'dir', 'file', 'id', 'title'}


    def __del__(self):
        if not Oasys.THIS._connection:
            return

        Oasys.THIS._connection.destructor(self.__class__.__name__, self._handle)


    def __getattr__(self, name):
# If one of the properties we define then get it
        if name in Model._props:
            return Oasys.THIS._connection.instanceGetter(self.__class__.__name__, self._handle, name)

        raise AttributeError


    def __setattr__(self, name, value):
# If one of the properties we define then set it
        if name in Model._props:
            Oasys.THIS._connection.instanceSetter(self.__class__.__name__, self._handle, name, value)
            return

# Set the property locally
        self.__dict__[name] = value


# Static methods
    def Exists(model_number):
        """
        Checks if a model exists

        Parameters
        ----------
        model_number : integer
            The number of the model you want to check the existence of

        Returns
        -------
        bool
            True if the model exists, otherwise False
        """
        return Oasys.THIS._connection.classMethod(__class__.__name__, "Exists", model_number)

    def GetFromID(model_number):
        """
        Returns the Model object for a model ID or None if model does not exist

        Parameters
        ----------
        model_number : integer
            number of the model you want the Model object for

        Returns
        -------
        Model
            Model object (or None if model does not exist)
        """
        return Oasys.THIS._connection.classMethod(__class__.__name__, "GetFromID", model_number)

    def HighestID():
        """
        Returns the ID of the highest model currently being used

        Returns
        -------
        int
            ID of highest model currently being used
        """
        return Oasys.THIS._connection.classMethod(__class__.__name__, "HighestID")

    def Read(filename, filetype=Oasys.gRPC.defaultArg):
        """
        Reads in a new model

        Parameters
        ----------
        filename : string
            Filename you want to read
        filetype : integer
            Optional. Filetypes you want to read. Can be bitwise OR of Model.THF, 
            Model.XTF, Model.LSDA, Model.ASCII, Model.ZTF and Model.ALL_FILES. 
            If omitted all available files will be read

        Returns
        -------
        Model
            Model object (or None if error)
        """
        return Oasys.THIS._connection.classMethod(__class__.__name__, "Read", filename, filetype)

    def Total():
        """
        Returns the total number of models

        Returns
        -------
        int
            integer
        """
        return Oasys.THIS._connection.classMethod(__class__.__name__, "Total")



# Instance methods
    def ClearFlag(self, flag, entity_type, item, end=Oasys.gRPC.defaultArg):
        """
        Clears a defined flag on an internal (or external) item(s) of type of entity_type in the model

        Parameters
        ----------
        flag : Flag
            The flag you want to clear
        entity_type : integer
            The Entity type that the defined flag will be cleared on
        item : integer
            If +ive: The internal item number starting from 1. If -ive: The external item label
        end : integer
            Optional. To unflag range of items, specify an optional end of range. Unflags items from item to range

        Returns
        -------
        bool
            True if the flag is successfully cleared on the item, otherwise False
        """
        return Oasys.THIS._connection.instanceMethod(self.__class__.__name__, self._handle, "ClearFlag", flag, entity_type, item, end)

    def Delete(self):
        """
        Deletes a model
        Do not use the Model object after calling this method

        Returns
        -------
        bool
            True if the model sucessfully deleted, otherwise False
        """
        return Oasys.THIS._connection.instanceMethod(self.__class__.__name__, self._handle, "Delete")

    def FlagAll(self, flag, entity_type):
        """
        Sets a defined flag on all of items of type of entity_type in the model

        Parameters
        ----------
        flag : Flag
            The flag you want to set
        entity_type : integer
            The Entity type that the defined flag will be set on

        Returns
        -------
        bool
            True if the flag is successfully set on all the items, otherwise False
        """
        return Oasys.THIS._connection.instanceMethod(self.__class__.__name__, self._handle, "FlagAll", flag, entity_type)

    def Flagged(self, flag, entity_type, item):
        """
        Checks if a defined flag is set on an internal (or external) item of type of entity_type in the model

        Parameters
        ----------
        flag : Flag
            The flag you want to check
        entity_type : integer
            The Entity type to check
        item : integer
            If +ive: The internal item number starting from 1. If -ive: The external item label

        Returns
        -------
        bool
            True if the flag is set, False if the flag is not set
        """
        return Oasys.THIS._connection.instanceMethod(self.__class__.__name__, self._handle, "Flagged", flag, entity_type, item)

    def GetDataFlagged(self, flag, data_comp, int_pnt=Oasys.gRPC.defaultArg, extra=Oasys.gRPC.defaultArg):
        """
        Gets curve objects for a data component for relevant items that are flagged with a specified flag in the model. 
        Some data components are valid for different entity types (e.g. SXX). If the same flag is set on items of different entity types, data is returned for all relevant, flagged entity types. 
        To return the same data for multiple items of the same type, it will be much faster if you flag all items you want data for, and do a single call to GetDataFlagged(). 
        The curves are ordered by type, then by the ascending internal index of the items. Use curve properties to identify which curve is which.
        If the data is not available in the model for a flagged item, or not available for the selected integration points or extra value, a curve is not returned.
        You can use QueryDataPresent() to check if the data is available.
        It is recommended that you check the number of curves returned. This can be compared with the number of flagged entities, 
        see GetNumberFlagged().
        If the data is generally available in the model, but not for the specific flagged item, a "None curve" which contains no x-y data values is returned.
        For example, a specific shell may have fewer integration points than MAX_INT for all shells, a "null curve" would be returned for the higher integration points.

        Parameters
        ----------
        flag : Flag
            The flag to use. For model data, use 0 to define a None "padding" argument
        data_comp : integer
            The Data Component to extract
        int_pnt : dict
            Optional. The integration points to extract. This argument is ignored when the entity type is not SOLID, SHELL, THICK_SHELL or BEAM.
            An integer specifies the integration point to extract:
            For SOLIDs: value between 0 for Average/Centre and 8. (Defaults to Average/Centre).
            For SHELLs and THICK_SHELLs: value between 1 and # integration points, or codes TOP, MIDDLE, BOTTOM. (Defaults to MIDDLE integration point). 
            For integrated BEAMs: value between 1 and # integration points. (Defaults to integration point 1).
            Use 0 to define a None "padding" argument, then uses the default integration point
        extra : integer
            Optional. The extra component id for SOLIDs, SHELLs, THICK_SHELLs or BEAMs

        Returns
        -------
        list
            List of Curve objects
        """
        return Oasys.THIS._connection.instanceMethod(self.__class__.__name__, self._handle, "GetDataFlagged", flag, data_comp, int_pnt, extra)

    def GetInternalID(self, entity_type, item):
        """
        Gets the internal ID of external item of type entity_type in the model

        Parameters
        ----------
        entity_type : integer
            The Entity type of the item
        item : integer
            The external item number

        Returns
        -------
        int
            Integer internal ID (starting from 1) with reference to the entity_type code. Returns integer internal ID of 0 if item cannot be found
        """
        return Oasys.THIS._connection.instanceMethod(self.__class__.__name__, self._handle, "GetInternalID", entity_type, item)

    def GetLabel(self, entity_type, item):
        """
        Gets the external label of internal item of type entity_type in the model

        Parameters
        ----------
        entity_type : integer
            The Entity type of the item
        item : integer
            The internal item number starting from 1

        Returns
        -------
        int
            Integer external ID (or 0 if there is an error, or the internal ID if there are no external IDs)
        """
        return Oasys.THIS._connection.instanceMethod(self.__class__.__name__, self._handle, "GetLabel", entity_type, item)

    def GetLabelFromName(self, entity_type, name):
        """
        Gets the external label from the database history name name of type entity_type in the model. 
        This is quicker if you use parent entity type codes (e.g. Entity.WELD rather than Entity.WELD_CONSTRAINED)

        Parameters
        ----------
        entity_type : integer
            The Entity type of the item
        name : string
            The name of the item. If only the first part of the name is given, it must be unambiguous

        Returns
        -------
        int
            Integer external ID of the first matching name (or 0 if there is an error)
        """
        return Oasys.THIS._connection.instanceMethod(self.__class__.__name__, self._handle, "GetLabelFromName", entity_type, name)

    def GetName(self, entity_type, item):
        """
        Gets the database history name of an internal (or external) item of type entity_type in the model

        Parameters
        ----------
        entity_type : integer
            The Entity type of the item
        item : integer
            If +ive: The internal item number starting from 1. If -ive: The external item label

        Returns
        -------
        str
            String containing the database history name (or None if not available)
        """
        return Oasys.THIS._connection.instanceMethod(self.__class__.__name__, self._handle, "GetName", entity_type, item)

    def GetNumberFlagged(self, flag, entity_type=Oasys.gRPC.defaultArg):
        """
        Gets the number of entities flagged with a requested flag in the model

        Parameters
        ----------
        flag : Flag
            The flag you want to check
        entity_type : integer
            Optional. If specified, the Entity type to look at. If not specified, all types are looked at

        Returns
        -------
        int
            Integer number
        """
        return Oasys.THIS._connection.instanceMethod(self.__class__.__name__, self._handle, "GetNumberFlagged", flag, entity_type)

    def GetNumberOf(self, entity_type):
        """
        Gets the number of entities of a requested type in the model

        Parameters
        ----------
        entity_type : integer
            The Entity type that you want to know the number of

        Returns
        -------
        int
            Integer number
        """
        return Oasys.THIS._connection.instanceMethod(self.__class__.__name__, self._handle, "GetNumberOf", entity_type)

    def QueryDataPresent(self, data_comp, entity_type=Oasys.gRPC.defaultArg, int_pnt=Oasys.gRPC.defaultArg, extra=Oasys.gRPC.defaultArg):
        """
        Checks if a data component data_comp for a given entity is present in a model's database.
        For SOLIDs, SHELLs, THICK_SHELLs and BEAMs the integration point and extra component ID can also be checked.
        This will show if curves for any flagged items of this type will be returned for GetDataFlagged(). 
        Note, it does not check if the data component is valid, for example a specific shell may have fewer integration points than MAX_INT for all shells, 
        so curves returned for GetDataFlagged() may still be "None" with no x-y data

        Parameters
        ----------
        data_comp : integer
            The Data Component to check
        entity_type : integer
            Optional. The Entity type to check. This argument can only be omitted when checking for global model data
        int_pnt : dict
            Optional. The integration points to check. This argument is ignored if the entity type is not SOLID, SHELL, THICK_SHELL or BEAM.
            An integer specifies the integration point to check:
            For SOLIDs: value between 0 for Average/Centre and 8. (Defaults to Average/Centre).
            For SHELLs and THICK_SHELLs: value between 1 and # integration points, or codes TOP, MIDDLE, BOTTOM. (Defaults to MIDDLE integration point). 
            For integrated BEAMs: value between 1 and # integration points. (Defaults to integration point 1).
            Use 0 to define a None "padding" argument, then checks the default integration point
        extra : integer
            Optional. The extra component id for SOLIDs, SHELLs, THICK_SHELLs or BEAMs

        Returns
        -------
        bool
            True if data is present, otherwise False
        """
        return Oasys.THIS._connection.instanceMethod(self.__class__.__name__, self._handle, "QueryDataPresent", data_comp, entity_type, int_pnt, extra)

    def SetFlag(self, flag, entity_type, item, end=Oasys.gRPC.defaultArg):
        """
        Sets a defined flag on an internal (or external) item(s) of type of entity_type in the model

        Parameters
        ----------
        flag : Flag
            The flag you want to set
        entity_type : integer
            The Entity type that the defined flag will be set on
        item : integer
            If +ive: The internal item number starting from 1. If -ive: The external item label
        end : integer
            Optional. To flag range of items, specify an optional end of range. Flags items from item to range

        Returns
        -------
        bool
            True if the flag is successfully set on the item, otherwise False
        """
        return Oasys.THIS._connection.instanceMethod(self.__class__.__name__, self._handle, "SetFlag", flag, entity_type, item, end)

    def UnflagAll(self, flag, entity_type):
        """
        Unsets a defined flag flag on all of items of type of entity_type in the model

        Parameters
        ----------
        flag : Flag
            The flag you want to unset
        entity_type : integer
            The Entity type that the defined flag will be unset on

        Returns
        -------
        bool
            True if the flag is successfully unset on all the items, otherwise False
        """
        return Oasys.THIS._connection.instanceMethod(self.__class__.__name__, self._handle, "UnflagAll", flag, entity_type)




import Oasys.gRPC


# Metaclass for static properties and constants
class UnitsType(type):
    _consts = {'ACCELERATION', 'AREA', 'CONDUCTIVITY', 'CURRENT', 'DENSITY', 'DISPLACEMENT', 'ELECTRIC_FIELD_VECTOR', 'ENERGY', 'ENERGY_DENSITY', 'FLUX', 'FORCE', 'FORCE_WIDTH', 'FREQUENCY', 'LENGTH', 'MAGNETIC_FLUX_VECTOR', 'MASS', 'MASS_FLOW', 'MOMENT', 'MOMENTUM', 'MOMENT_WIDTH', 'NONE', 'POWER', 'PRESSURE', 'Q_CRITERION', 'ROTATION', 'ROTATIONAL_ACCELERATION', 'ROTATIONAL_VELOCITY', 'STRAIN', 'STRESS', 'TEMPERATURE', 'THERMAL_DIFFUSIVITY', 'TIME', 'UNKNOWN', 'VECTOR_POTENTIAL', 'VELOCITY', 'VISCOSITY', 'VOLUME', 'VORTICITY', 'WORK'}

    def __getattr__(cls, name):
        if name in UnitsType._consts:
            return Oasys.THIS._connection.classGetter(cls.__name__, name)

        raise AttributeError


class Units(Oasys.gRPC.OasysItem, metaclass=UnitsType):


    def __del__(self):
        if not Oasys.THIS._connection:
            return

        Oasys.THIS._connection.destructor(self.__class__.__name__, self._handle)


    def __getattr__(self, name):
        raise AttributeError


    def __setattr__(self, name, value):
# Set the property locally
        self.__dict__[name] = value


# Static methods
    def USER(mass, time, length, angle, temperature, current=Oasys.gRPC.defaultArg):
        """
        Setup a user defined UNIT

        Parameters
        ----------
        mass : float
            Power for mass dimensions
        time : float
            Power for time dimensions
        length : float
            Power for length dimensions
        angle : float
            Power for angle dimensions
        temperature : float
            Power for temperature dimensions
        current : float
            Optional. Power for current dimensions

        Returns
        -------
        int
            integer
        """
        return Oasys.THIS._connection.classMethod(__class__.__name__, "USER", mass, time, length, angle, temperature, current)




import os
import subprocess
import time
import Oasys.gRPC


_connection = None
_debug      = False


def start(abspath, args=None, port=50052, memory=25, license=None, debug=False, batch=False):
    if debug:
        print("Starting {}, listening on port {}".format(abspath, port))

    environ = os.environ.copy()
    if license:
        environ[ARUP_LICENSE_PATH] = license

    pargs = [abspath, "-grpc={}".format(port)]
    if args:
        pargs.extend(args)
    if batch:
        pargs.append("-batch")

    subprocess.Popen(pargs, env=environ)

# Sleep to give time for T/HIS to start
    time.sleep(5)

    return connect(port, memory, 'localhost', debug)


def connect(port=50052, memory=25, hostname='localhost', debug=False):
    if debug:
        print("Called connect with port {} and memory {}".format(port, memory))

    Oasys.THIS._debug      = debug
    Oasys.THIS._connection = Oasys.gRPC.Connection("T/HIS", port, memory, hostname, debug)
    return Oasys.THIS._connection


def disconnect(connection):
    if Oasys.THIS._debug:
        print("Called disconnect")

    connection.finalise()
    Oasys.THIS._connection = None


def terminate(connection):
    if Oasys.THIS._debug:
        print("Called terminate")

    connection.terminate()
    Oasys.THIS._connection = None


def createInstance(t, h):
# Classes in T/HIS
    if t == "Colour":
        instance = object.__new__(Oasys.THIS.Colour)
    elif t == "Component":
        instance = object.__new__(Oasys.THIS.Component)
    elif t == "Curve":
        instance = object.__new__(Oasys.THIS.Curve)
    elif t == "Datum":
        instance = object.__new__(Oasys.THIS.Datum)
    elif t == "Entity":
        instance = object.__new__(Oasys.THIS.Entity)
    elif t == "Graph":
        instance = object.__new__(Oasys.THIS.Graph)
    elif t == "Group":
        instance = object.__new__(Oasys.THIS.Group)
    elif t == "Include":
        instance = object.__new__(Oasys.THIS.Include)
    elif t == "LineStyle":
        instance = object.__new__(Oasys.THIS.LineStyle)
    elif t == "LineWidth":
        instance = object.__new__(Oasys.THIS.LineWidth)
    elif t == "Model":
        instance = object.__new__(Oasys.THIS.Model)
    elif t == "Operate":
        instance = object.__new__(Oasys.THIS.Operate)
    elif t == "Options":
        instance = object.__new__(Oasys.THIS.Options)
    elif t == "Page":
        instance = object.__new__(Oasys.THIS.Page)
    elif t == "Read":
        instance = object.__new__(Oasys.THIS.Read)
    elif t == "Symbol":
        instance = object.__new__(Oasys.THIS.Symbol)
    elif t == "UnitSystem":
        instance = object.__new__(Oasys.THIS.UnitSystem)
    elif t == "Units":
        instance = object.__new__(Oasys.THIS.Units)
    elif t == "Utils":
        instance = object.__new__(Oasys.THIS.Utils)
    elif t == "Window":
        instance = object.__new__(Oasys.THIS.Window)
    elif t == "Workflow":
        instance = object.__new__(Oasys.THIS.Workflow)

# Generic object
    elif t == "ItemObject":
        instance = object.__new__(Oasys.THIS.ItemObject)

# Unsupported
    else:
        raise NotImplementedError("Instance type '{}' not implemented".format(t))

    instance.__dict__['_handle']  = h;
    instance.__dict__['_objtype'] = t;

    return instance




import Oasys.gRPC


# Metaclass for static properties and constants
class LineStyleType(type):
    _consts = {'DASH', 'DASH2', 'DASH3', 'DASH4', 'DASH5', 'DASH6', 'NONE', 'SOLID'}

    def __getattr__(cls, name):
        if name in LineStyleType._consts:
            return Oasys.THIS._connection.classGetter(cls.__name__, name)

        raise AttributeError


class LineStyle(Oasys.gRPC.OasysItem, metaclass=LineStyleType):


    def __del__(self):
        if not Oasys.THIS._connection:
            return

        Oasys.THIS._connection.destructor(self.__class__.__name__, self._handle)


    def __getattr__(self, name):
        raise AttributeError


    def __setattr__(self, name, value):
# Set the property locally
        self.__dict__[name] = value



import Oasys.gRPC


# Metaclass for static properties and constants
class ComponentType(type):
    _consts = {'A', 'AA', 'AC', 'ADT', 'AFM', 'AFX', 'AFY', 'AFZ', 'AIE', 'ALD', 'ALS', 'ALT', 'AM', 'AME', 'APR', 'AR', 'ARS', 'ATE', 'AVM', 'AVX', 'AVY', 'AVZ', 'AX', 'AY', 'AZ', 'BA', 'BACE', 'BDDX', 'BDDY', 'BDDZ', 'BDMA', 'BDMS', 'BDMT', 'BDNA', 'BDNS', 'BDNT', 'BDSX', 'BDSY', 'BDSZ', 'BDT', 'BDTX', 'BDTY', 'BDTZ', 'BDX', 'BDY', 'BDZ', 'BEAX', 'BED', 'BEP', 'BES', 'BET', 'BEX', 'BFDM', 'BFDX', 'BFDY', 'BFDZ', 'BFX', 'BFY', 'BFZ', 'BIE', 'BMXX', 'BMY1', 'BMY2', 'BMYY', 'BMZ1', 'BMZ2', 'BMZZ', 'BPE1', 'BPE2', 'BRXX', 'BRY', 'BRY1', 'BRY2', 'BRZ', 'BRZ1', 'BRZ2', 'BSAX', 'BSXX', 'BSXY', 'BSZX', 'BV', 'BX', 'BY', 'BZ', 'C10', 'C20', 'C30', 'CENA', 'CENB', 'CENM', 'CENS', 'CFL', 'CFM', 'CFMA', 'CFMB', 'CFMS', 'CFRI', 'CFX', 'CFXA', 'CFXB', 'CFXS', 'CFY', 'CFYA', 'CFYB', 'CFYS', 'CFZ', 'CFZA', 'CFZB', 'CFZS', 'CMA', 'CMB', 'CMM', 'CMS', 'CMX', 'CMXA', 'CMXB', 'CMXS', 'CMY', 'CMYA', 'CMYB', 'CMYS', 'CMZ', 'CMZA', 'CMZB', 'CMZS', 'CNM', 'COUT', 'CTEN', 'CV', 'CX', 'CY', 'CZ', 'D11', 'D12', 'D13', 'D21', 'D22', 'D23', 'D31', 'D32', 'D33', 'DE', 'DFP', 'DFR', 'DFX', 'DFY', 'DFZ', 'DM', 'DMP', 'DP', 'DPDT', 'DPFX', 'DPFY', 'DPFZ', 'DPIE', 'DPPR', 'DPTE', 'DR', 'DRCE', 'DRCO', 'DRDT', 'DRKE', 'DRMX', 'DVO', 'DX', 'DXDT', 'DY', 'DYDT', 'DZ', 'DZDT', 'EAV', 'EBA', 'EBC', 'EBV', 'ECC', 'ECCE', 'ECCT', 'ECDC', 'ECDE', 'ECDT', 'ECDV', 'ECE', 'ECI', 'ECJH', 'ECM', 'ECM1', 'ECM2', 'ECM3', 'ECME', 'ECP', 'ECRA', 'ECRC', 'ECRD', 'ECRJ', 'ECRR', 'ECT', 'ECV', 'ECX', 'ECY', 'ECZ', 'EDEN', 'EFM', 'EFS', 'EFX', 'EFY', 'EFZ', 'EICT', 'EIV', 'EMAX', 'EMIN', 'EMS', 'EN', 'ENE', 'EPL', 'EPS', 'ERD', 'ERSC', 'ERVC', 'ERVM', 'ERXX', 'ERXY', 'ERYY', 'ERYZ', 'ERZX', 'ERZZ', 'ESE', 'EV', 'EVON', 'EXX', 'EXY', 'EYY', 'EYZ', 'EZX', 'EZZ', 'FM', 'FN', 'FPM', 'FPX', 'FPY', 'FPZ', 'FR', 'FVM', 'FVX', 'FVY', 'FVZ', 'FX', 'FY', 'FZ', 'GA', 'GAM', 'GAS', 'GCM', 'GDE', 'GDIE', 'GDKE', 'GDT', 'GEHG', 'GEIE', 'GEKE', 'GEN', 'GER', 'GEW', 'GHG', 'GI1', 'GI11', 'GI12', 'GI13', 'GI2', 'GI21', 'GI22', 'GI23', 'GI3', 'GI31', 'GI32', 'GI33', 'GIE', 'GIR', 'GJE', 'GKE', 'GKR', 'GMADD', 'GMASS', 'GMDE', 'GMEE', 'GMPE', 'GMX', 'GMY', 'GMZ', 'GP11', 'GP12', 'GP13', 'GP21', 'GP22', 'GP23', 'GP31', 'GP32', 'GP33', 'GPM', 'GRBE', 'GRFM', 'GRFX', 'GRFY', 'GRFZ', 'GSDE', 'GSF', 'GSIE', 'GSPE', 'GSTP', 'GSWE', 'GTE', 'GTER', 'GTZC', 'GVX', 'GVY', 'GVZ', 'GXCM', 'GYCM', 'GZCM', 'HCE', 'HTC', 'IE', 'IFE', 'IN', 'JHR', 'LAX', 'LAY', 'LAZ', 'LDX', 'LDY', 'LDZ', 'LE', 'LFM', 'LFX', 'LFY', 'LFZ', 'LK', 'LKE', 'LMX', 'LMY', 'LMZ', 'LOFM', 'LOFX', 'LOFY', 'LOFZ', 'LRAX', 'LRAY', 'LRAZ', 'LRDX', 'LRDY', 'LRDZ', 'LRVX', 'LRVY', 'LRVZ', 'LS', 'LVX', 'LVY', 'LVZ', 'MAF', 'MASS', 'MAV', 'MIN', 'MJH', 'ML', 'MM', 'MME', 'MOF', 'MOU', 'MOV', 'MP', 'MPM', 'MPX', 'MPY', 'MPZ', 'MSR', 'MUR', 'MVM', 'MVX', 'MVY', 'MVZ', 'MX', 'MXR', 'MY', 'MZ', 'NP', 'NSC', 'OCV', 'OHE', 'OHP', 'OIE', 'OTE', 'OU', 'PA', 'PEMAG', 'PHA', 'PHD', 'PHDT', 'PHS', 'PHT', 'PJHE', 'PKIN', 'PLFM', 'PLFX', 'PLFY', 'PLFZ', 'PL_AN', 'PL_FT', 'PL_SL', 'PL_SR', 'PMAG', 'POW', 'PP', 'PPLA', 'PR', 'PRN', 'PRP', 'PR_FI', 'PSA', 'PSD', 'PSDT', 'PSS', 'PST', 'PTE', 'PVO', 'QC', 'R10', 'R20', 'R30', 'RAM', 'RAX', 'RAY', 'RAZ', 'RBC', 'RCT', 'RDM', 'RDX', 'RDY', 'RDZ', 'RFM', 'RFX', 'RFXY', 'RFY', 'RFZ', 'RHE', 'RHP', 'RMX', 'RMXY', 'RMY', 'RPR', 'RQX', 'RQY', 'RR0', 'RT_F', 'RT_FP', 'RT_P', 'RUN', 'RV', 'RVM', 'RVX', 'RVY', 'RVZ', 'S', 'SA', 'SAV', 'SB_F', 'SB_FS', 'SB_L', 'SB_S', 'SFP', 'SFR', 'SFX', 'SFY', 'SFZ', 'SHC', 'SHX', 'SM', 'SMAX', 'SMIN', 'SMS', 'SO', 'SOC', 'SOF', 'SOM', 'SOS', 'SOX', 'SPC_FM', 'SPC_FX', 'SPC_FY', 'SPC_FZ', 'SPC_MM', 'SPC_MX', 'SPC_MY', 'SPC_MZ', 'SPC_RF', 'SPC_RMF', 'SPC_XMF', 'SPC_XTF', 'SPC_YMF', 'SPC_YTF', 'SPC_ZMF', 'SPC_ZTF', 'SP_E', 'SP_EN', 'SP_F', 'SP_FE', 'SP_FX', 'SP_FY', 'SP_FZ', 'SP_M', 'SP_MR', 'SP_MX', 'SP_MY', 'SP_MZ', 'SP_R', 'SR_B1', 'SR_B2', 'SR_F', 'SR_N', 'SR_P', 'SR_S', 'SR_W', 'STR', 'SVON', 'SW_AREA', 'SW_BF', 'SW_F', 'SW_FAIL', 'SW_FF', 'SW_LE', 'SW_MF', 'SW_MM', 'SW_NF', 'SW_S', 'SW_SF', 'SW_TIME', 'SW_TO', 'SXX', 'SXY', 'SYY', 'SYZ', 'SZX', 'SZZ', 'TAA', 'TBOT', 'TC', 'TEE', 'TEH', 'TEM', 'TEMP', 'TFM', 'TFP', 'TFR', 'TFX', 'TFY', 'TFZ', 'THA', 'THD', 'THDT', 'THK', 'THS', 'THT', 'TKE', 'TNC', 'TOC', 'TOR', 'TPE', 'TRE', 'TSA', 'TSR', 'TTOP', 'TVO', 'UN', 'VC', 'VC2', 'VC3', 'VEL', 'VM', 'VOL', 'VT', 'VTM', 'VTX', 'VTY', 'VTZ', 'VX', 'VY', 'VZ', 'X', 'XSEC_A', 'XSEC_CX', 'XSEC_CY', 'XSEC_CZ', 'XSEC_FM', 'XSEC_FX', 'XSEC_FY', 'XSEC_FZ', 'XSEC_MM', 'XSEC_MX', 'XSEC_MY', 'XSEC_MZ', 'Y', 'Z'}

    def __getattr__(cls, name):
        if name in ComponentType._consts:
            return Oasys.THIS._connection.classGetter(cls.__name__, name)

        raise AttributeError


class Component(Oasys.gRPC.OasysItem, metaclass=ComponentType):


    def __del__(self):
        if not Oasys.THIS._connection:
            return

        Oasys.THIS._connection.destructor(self.__class__.__name__, self._handle)


    def __getattr__(self, name):
        raise AttributeError


    def __setattr__(self, name, value):
# Set the property locally
        self.__dict__[name] = value



import Oasys.gRPC


# Metaclass for static properties and constants
class IncludeType(type):
    _consts = {'NATIVE', 'UNIX', 'WINDOWS'}

    def __getattr__(cls, name):
        if name in IncludeType._consts:
            return Oasys.THIS._connection.classGetter(cls.__name__, name)

        raise AttributeError


class Include(Oasys.gRPC.OasysItem, metaclass=IncludeType):


    def __del__(self):
        if not Oasys.THIS._connection:
            return

        Oasys.THIS._connection.destructor(self.__class__.__name__, self._handle)


    def __getattr__(self, name):
        raise AttributeError


    def __setattr__(self, name, value):
# Set the property locally
        self.__dict__[name] = value



import Oasys.gRPC


# Metaclass for static properties and constants
class UtilsType(type):

    def __getattr__(cls, name):

        raise AttributeError


class Utils(Oasys.gRPC.OasysItem, metaclass=UtilsType):


    def __del__(self):
        if not Oasys.THIS._connection:
            return

        Oasys.THIS._connection.destructor(self.__class__.__name__, self._handle)


    def __getattr__(self, name):
        raise AttributeError


    def __setattr__(self, name, value):
# Set the property locally
        self.__dict__[name] = value


# Static methods
    def Ascii85Encode(data, length=Oasys.gRPC.defaultArg):
        """
        Encodes an ASCII85 encoded string. This enables binary data to be represented by ASCII characters using five ASCII characters
        to represent four bytes of binary data (making the encoded size 1/4 larger than the original). By doing this binary data can be stored in
        JavaScript strings. Note that the method used by THIS to encode and decode strings differs from the standard ASCII85 encoding as that uses the
        ASCII characters ", ' and \ which cannot be used in JavaScript strings as they have special meanings. The method in THIS uses
        0-84 are !-u (ASCII codes 33-117) (i.e. 33 is added to it) with the following exceptions
        v is used instead of " (ASCII code 118 instead of 34)
        w is used instead of ' (ASCII code 119 instead of 39)
        x is used instead of \ (ASCII code 120 instead of 92)
        If all five digits are 0 they are represented by a single character z instead of !!!!!

        Parameters
        ----------
        data : `ListBuffer `_
            `ListBuffer `_ containing the data
        length : integer
            Optional. Length of data in list buffer to encode. If omitted the whole array buffer will be encoded

        Returns
        -------
        str
            string
        """
        return Oasys.THIS._connection.classMethod(__class__.__name__, "Ascii85Encode", data, length)

    def CallPromiseHandlers():
        """
        Manually call any promise handlers/callbacks in the job queue

        Returns
        -------
        None
            no return value
        """
        return Oasys.THIS._connection.classMethod(__class__.__name__, "CallPromiseHandlers")

    def CheckinLicense(feature):
        """
        Checks a license for a feature back in

        Parameters
        ----------
        feature : string
            feature to check license back in for

        Returns
        -------
        None
            no return value
        """
        return Oasys.THIS._connection.classMethod(__class__.__name__, "CheckinLicense", feature)

    def CheckoutLicense(feature):
        """
        Checks out a license for a feature

        Parameters
        ----------
        feature : string
            feature to check license for

        Returns
        -------
        bool
            True if license available, False if not
        """
        return Oasys.THIS._connection.classMethod(__class__.__name__, "CheckoutLicense", feature)

    def GarbageCollect():
        """
        Forces garbage collection to be done. This should not normally need to be called
        but in exceptional circumstances it can be called to ensure that garbage collection is done to
        return memory

        Returns
        -------
        None
            no return value
        """
        return Oasys.THIS._connection.classMethod(__class__.__name__, "GarbageCollect")

    def HTMLBrowser():
        """
        Returns the path to the default HTML browser

        Returns
        -------
        str
            string of the path
        """
        return Oasys.THIS._connection.classMethod(__class__.__name__, "HTMLBrowser")

    def HiResTimer():
        """
        A high resolution timer that can be used to time how long things take.
        The first time this is called the timer will start and return 0. Subsequent calls will return
        the time in nanoseconds since the first call. Note that the timer will almost certainly not have
        1 nanosecond precision but, depending on the platform, should should have a resolution of at least 1 microsecond.
        The resolution can be found by using Utils.TimerResolution()

        Returns
        -------
        """
        return Oasys.THIS._connection.classMethod(__class__.__name__, "HiResTimer")

    def PdfReader():
        """
        Returns the path to the executable of the default pdf reader

        Returns
        -------
        str
            string of the path
        """
        return Oasys.THIS._connection.classMethod(__class__.__name__, "PdfReader")

    def TimerResolution():
        """
        Returns the resolution (precision) of the Utils.HiResTimer() timer in nanoseconds

        Returns
        -------
        """
        return Oasys.THIS._connection.classMethod(__class__.__name__, "TimerResolution")

    def Version():
        """
        Returns the version number

        Returns
        -------
        float
            real
        """
        return Oasys.THIS._connection.classMethod(__class__.__name__, "Version")




import Oasys.gRPC


# Metaclass for static properties and constants
class WorkflowType(type):
    _consts = {'UNIT_SYSTEM_NONE', 'UNIT_SYSTEM_U1', 'UNIT_SYSTEM_U2', 'UNIT_SYSTEM_U3', 'UNIT_SYSTEM_U4', 'UNIT_SYSTEM_U5', 'UNIT_SYSTEM_U6'}

    def __getattr__(cls, name):
        if name in WorkflowType._consts:
            return Oasys.THIS._connection.classGetter(cls.__name__, name)

        raise AttributeError


class Workflow(Oasys.gRPC.OasysItem, metaclass=WorkflowType):


    def __del__(self):
        if not Oasys.THIS._connection:
            return

        Oasys.THIS._connection.destructor(self.__class__.__name__, self._handle)


    def __getattr__(self, name):
        raise AttributeError


    def __setattr__(self, name, value):
# Set the property locally
        self.__dict__[name] = value


# Static methods
    def ModelIdFromIndex(model_index, workflow_name=Oasys.gRPC.defaultArg):
        """
        Returns the id of a model selected by the user by index (starting at 0)

        Parameters
        ----------
        model_index : integer
            The index of the model to return the unit system for.
            If the workflow is run from the workflow menu and the name argument is not defined, it is the index in the list of
            models selected by the user.
            If the workflow is run from the workflow menu and the name argument is defined, it is the index of the model that
            has user data for the named workflow, out of the list of models selected by the user.
            If the workflow is run from REPORTER, it is the index in the list of all the models loaded in the session that 
            have user data for the named workflow
        workflow_name : string
            Optional. The workflow name to return the model id for

        Returns
        -------
        int
            integer
        """
        return Oasys.THIS._connection.classMethod(__class__.__name__, "ModelIdFromIndex", model_index, workflow_name)

    def ModelUnitSystemFromIndex(model_index, workflow_name=Oasys.gRPC.defaultArg):
        """
        Returns the unit system of a model selected by the user by index (starting at 0).
        Will be Workflow.UNIT_SYSTEM_NONE or
        Workflow.UNIT_SYSTEM_U1 or
        Workflow.UNIT_SYSTEM_U2 or
        Workflow.UNIT_SYSTEM_U3 or
        Workflow.UNIT_SYSTEM_U4 or
        Workflow.UNIT_SYSTEM_U5 or
        Workflow.UNIT_SYSTEM_U6

        Parameters
        ----------
        model_index : integer
            The index of the model to return the unit system for.
            If the workflow is run from the workflow menu and the name argument is not defined, it is the index in the list of
            models selected by the user.
            If the workflow is run from the workflow menu and the name argument is defined, it is the index of the model that
            has user data for the named workflow, out of the list of models selected by the user.
            If the workflow is run from REPORTER, it is the index in the list of all the models loaded in the session that 
            have user data for the named workflow
        workflow_name : string
            Optional. The workflow name to return the unit system for

        Returns
        -------
        int
            integer
        """
        return Oasys.THIS._connection.classMethod(__class__.__name__, "ModelUnitSystemFromIndex", model_index, workflow_name)

    def ModelUserDataBuildFromIndex(model_index, workflow_name=Oasys.gRPC.defaultArg):
        """
        Returns the build number of the program that was used to write out the
        user data of a model for the selected workflow by index (starting at 0)

        Parameters
        ----------
        model_index : integer
            The index of the model to return the program build number for
        workflow_name : string
            Optional. The workflow name to return the build number for. This is required when a PRIMER item is
            generated from REPORTER. If it is not specified the build number for the first user data associated with 
            the model is returned (this is the 'normal' case where a user launches a workflow from the workflow
            menu)

        Returns
        -------
        """
        return Oasys.THIS._connection.classMethod(__class__.__name__, "ModelUserDataBuildFromIndex", model_index, workflow_name)

    def ModelUserDataFromIndex(model_index, workflow_name=Oasys.gRPC.defaultArg):
        """
        Returns the user data associated with a model by index (starting at 0)

        Parameters
        ----------
        model_index : integer
            The index of the model to return the user data for.
            If the workflow is run from the workflow menu and the name argument is not defined, it is the index in the list of
            models selected by the user.
            If the workflow is run from the workflow menu and the name argument is defined, it is the index of the model that
            has user data for the named workflow, out of the list of models selected by the user.
            If the workflow is run from REPORTER, it is the index in the list of all the models loaded in the session that 
            have user data for the named workflow
        workflow_name : string
            Optional. The workflow name to return the user data for

        Returns
        -------
        dict
            Dict
        """
        return Oasys.THIS._connection.classMethod(__class__.__name__, "ModelUserDataFromIndex", model_index, workflow_name)

    def ModelUserDataProgramFromIndex(model_index, workflow_name=Oasys.gRPC.defaultArg):
        """
        Returns the name of the program that was used to write out the
        user data of a model for the selected workflow by index (starting at 0)

        Parameters
        ----------
        model_index : integer
            The index of the model to return the program name for
        workflow_name : string
            Optional. The workflow name to return the program name for. This is required when a PRIMER item is
            generated from REPORTER. If it is not specified the program name for the first user data associated with 
            the model is returned (this is the 'normal' case where a user launches a workflow from the workflow
            menu)

        Returns
        -------
        str
            string
        """
        return Oasys.THIS._connection.classMethod(__class__.__name__, "ModelUserDataProgramFromIndex", model_index, workflow_name)

    def ModelUserDataVersionFromIndex(model_index, workflow_name=Oasys.gRPC.defaultArg):
        """
        Returns the version of the program that was used to write out the
        user data of a model for the selected workflow by index (starting at 0)

        Parameters
        ----------
        model_index : integer
            The index of the model to return the program version for
        workflow_name : string
            Optional. The workflow name to return the version for. This is required when a PRIMER item is
            generated from REPORTER. If it is not specified the version for the first user data associated with 
            the model is returned (this is the 'normal' case where a user launches a workflow from the workflow
            menu)

        Returns
        -------
        """
        return Oasys.THIS._connection.classMethod(__class__.__name__, "ModelUserDataVersionFromIndex", model_index, workflow_name)

    def NumberOfSelectedModels(workflow_name=Oasys.gRPC.defaultArg):
        """
        Returns the number of models selected by the user

        Parameters
        ----------
        workflow_name : string
            Optional. The workflow name to return the number of models for.
            If it's not defined the number of models that were selected by the user on the workflow menu is returned.
            If it's defined and the workflow was run from the workflow menu, the number of models, out of the models selected by the user, that have data for the named workflow is returned.
            If it's defined and the workflow is run from REPORTER, the number of models, out of all the models loaded in the session, that have data for the named workflow is returned

        Returns
        -------
        int
            integer
        """
        return Oasys.THIS._connection.classMethod(__class__.__name__, "NumberOfSelectedModels", workflow_name)

    def Refresh():
        """
        Scan for fresh workflow data

        Returns
        -------
        None
            No return value
        """
        return Oasys.THIS._connection.classMethod(__class__.__name__, "Refresh")

    def WorkflowDefinitionFilename(workflow_name=Oasys.gRPC.defaultArg):
        """
        Returns the workflow definition filename

        Parameters
        ----------
        workflow_name : string
            Optional. The workflow name to return the workflow defintion filename for. This is required when a POST item is
            generated from REPORTER. If it is not specified the first workflow user data associated with 
            the model is returned (this is the 'normal' case where a user launches a workflow from the workflow
            menu)

        Returns
        -------
        str
            string
        """
        return Oasys.THIS._connection.classMethod(__class__.__name__, "WorkflowDefinitionFilename", workflow_name)

    def WriteToFile(user_data, output_filename, workflow_definition_filename, extra=Oasys.gRPC.defaultArg):
        """
        Writes a workflow to a JSON file. If the file already exists the workflow
        is added to the file (or updated if it is already in the file)

        Parameters
        ----------
        user_data : object
            Object containing user data required for the workflow
        output_filename : string
            Filename to write to
        workflow_definition_filename : string
            Filename of the workflow definition file
        extra : dict
            Optional. Extra workflow information

        Returns
        -------
        None
            No return value
        """
        return Oasys.THIS._connection.classMethod(__class__.__name__, "WriteToFile", user_data, output_filename, workflow_definition_filename, extra)




import Oasys.gRPC


def AllocateFlag():
    """
    Allocate a flag for use in the script. See also
    ReturnFlag() and
    Once allocated the flag is automatically cleared for all entity types and all the curves currently in T/HIS

    Returns
    -------
    int
        Flag (integer)
    """
    return Oasys.THIS._connection.functionCall("AllocateFlag")


def ClearFlag(flag):
    """
    Clears a flag on all curves and entity types

    Parameters
    ----------
    flag : Flag
        The flag to return

    Returns
    -------
    None
        No return value
    """
    return Oasys.THIS._connection.functionCall("ClearFlag", flag)


def DialogueInput(*command):
    """
    Execute one or more lines of command line dialogue input

    Parameters
    ----------
    *command : string
        Command to execute (as if it had been typed into the dialogue box)
        This argument can be repeated if required.
        Alternatively a single array argument containing the multiple values can be given

    Returns
    -------
    None
        No return value
    """
    return Oasys.THIS._connection.functionCall("DialogueInput", command)


def DialogueInputNoEcho(*command):
    """
    Execute one or more lines of command line dialogue input with no echo of commands to dialogue box

    Parameters
    ----------
    *command : string
        Command to execute (as if it had been typed into the dialogue box)
        This argument can be repeated if required.
        Alternatively a single array argument containing the multiple values can be given

    Returns
    -------
    None
        No return value
    """
    return Oasys.THIS._connection.functionCall("DialogueInputNoEcho", command)


def DisableGraphWindowUpdates():
    """
    Disable Graph Window updates

    Returns
    -------
    None
        No return value
    """
    return Oasys.THIS._connection.functionCall("DisableGraphWindowUpdates")


def DisableMenuUpdates():
    """
    Disable menu system updates

    Returns
    -------
    None
        No return value
    """
    return Oasys.THIS._connection.functionCall("DisableMenuUpdates")


def EnableGraphWindowUpdates():
    """
    Enable Graph Window updates

    Returns
    -------
    None
        No return value
    """
    return Oasys.THIS._connection.functionCall("EnableGraphWindowUpdates")


def EnableMenuUpdates():
    """
    Enable menu system updates

    Returns
    -------
    None
        No return value
    """
    return Oasys.THIS._connection.functionCall("EnableMenuUpdates")


def ErrorMessage(string):
    """
    Print an error message to the dialogue box adding a carriage return

    Parameters
    ----------
    string : Any valid javascript type
        The string/item that you want to print

    Returns
    -------
    None
        No return value
    """
    return Oasys.THIS._connection.functionCall("ErrorMessage", string)


def Execute(data):
    """
    Execute a program or script outside T/HIS and get the standard output and error streams

    Parameters
    ----------
    data : dict
        Execute data

    Returns
    -------
    dict
        Dict with properties
    """
    return Oasys.THIS._connection.functionCall("Execute", data)


def Exit(write_hook_interrupt=Oasys.gRPC.defaultArg):
    """
    Exit script

    Parameters
    ----------
    write_hook_interrupt : boolean
        Optional. If Exit() is called from a write_hook.js script, the first argument will be processed as in the following: 
        If the argument is provided and set to "true", it is used to interrupt the write out of the model, so that 
        the script exits without anything being written out. An argument value of "false" exits the script and allows 
        the model to be written out as normal. An example of this function's use in a Write Hook script can be found at 
        $OA_INSTALL/primer_library/scripts/hooks/example_write_hook.js

    Returns
    -------
    None
        No return value
    """
    return Oasys.THIS._connection.functionCall("Exit", write_hook_interrupt)


def GetCurrentDirectory():
    """
    Get the current working directory

    Returns
    -------
    str
        String containing current working directory
    """
    return Oasys.THIS._connection.functionCall("GetCurrentDirectory")


def GetFtcfVar(name):
    """
    Get the value of a FAST-TCF variable

    Parameters
    ----------
    name : string
        The FAST-TCF variable name (case independent)

    Returns
    -------
    str
        String containing variable value or None if variable does not exist
    """
    return Oasys.THIS._connection.functionCall("GetFtcfVar", name)


def GetInstallDirectory():
    """
    Get the directory in which executables are installed. This is the OA_INSTALL environment variable,
    or if that is not set the directory in which the current executable is installed. Returns None if not found

    Returns
    -------
    str
        string
    """
    return Oasys.THIS._connection.functionCall("GetInstallDirectory")


def GetPreferenceValue(program, name):
    """
    Get the Preference value with the given string in the any of 
    admin ("OA_ADMIN") or install ("OA_INSTALL") or home ("OA_HOME") directory oa_pref

    Parameters
    ----------
    program : string
        The program name string : Valid values are 'All', 'D3Plot', 'Primer', 'Reporter', 'Shell',
        'T/His'
    name : string
        The preference name string

    Returns
    -------
    str
        : String containing preference value or None if preference string is not present in any oa_pref. Also if none of the above environment variables are not present, then API simply returns null. While returning preference value, locked preference value in admin and then install oa_pref takes precedence over home oa_pref. If preference is not locked in any of these oa_pref, preference in home directory oa_pref is returned
    """
    return Oasys.THIS._connection.functionCall("GetPreferenceValue", program, name)


def GetStartInDirectory():
    """
    Get the directory passed to T/HIS by the -start_in command line argument

    Returns
    -------
    str
        String containing start_in directory or None if not set
    """
    return Oasys.THIS._connection.functionCall("GetStartInDirectory")


def Getenv(name):
    """
    Get the value of an environment variable

    Parameters
    ----------
    name : string
        The environment variable name

    Returns
    -------
    str
        String containing variable value or None if variable does not exist
    """
    return Oasys.THIS._connection.functionCall("Getenv", name)


def Message(string):
    """
    Print a message to the dialogue box adding a carriage return

    Parameters
    ----------
    string : Any valid javascript type
        The string/item that you want to print. If '\r' is added to the end of the string then
        instead of automatically adding a carriage return in the dialogue box, the next message will overwrite 
        the current one. This may be useful for giving feedback to the dialogue box when doing an operation

    Returns
    -------
    None
        No return value
    """
    return Oasys.THIS._connection.functionCall("Message", string)


def MilliSleep(time):
    """
    Pause execution of the script for time milliseconds. See also
    Sleep()

    Parameters
    ----------
    time : integer
        Number of milliseconds to pause for

    Returns
    -------
    None
        No return value
    """
    return Oasys.THIS._connection.functionCall("MilliSleep", time)


def NumberToString(number, width, pref_int=Oasys.gRPC.defaultArg):
    """
    Formats a number to a string with the specified width

    Parameters
    ----------
    number : integer/float
        The number you want to format
    width : integer
        The width of the string you want to format it to (must be less than 80)
    pref_int : boolean
        Optional. By default only integer values inside the single precision 32 bit signed integer limit of approximately
        +/-2e9 are formatted as integers, all other numeric values are formatted as floats. With this argument set to TRUE then
        integer values up to the mantissa precision of a 64 bit float, approximately +/-9e15, will also be formatted as integers

    Returns
    -------
    str
        String containing the number
    """
    return Oasys.THIS._connection.functionCall("NumberToString", number, width, pref_int)


def OpenManual(program, page):
    """
    Open the Oasys manuals at a requested page

    Parameters
    ----------
    program : string
        The program manual to open. Can be "primer", "d3plot" or "this"
    page : string
        The page to open in the manual, e.g. "running-this.html"

    Returns
    -------
    bool
        True if successful, False if not
    """
    return Oasys.THIS._connection.functionCall("OpenManual", program, page)


def Plot():
    """
    Updates all the T/HIS graphs

    Returns
    -------
    None
        No return value
    """
    return Oasys.THIS._connection.functionCall("Plot")


def Print(string):
    """
    Print a string to stdout. Note that a carriage return is not added

    Parameters
    ----------
    string : Any valid javascript type
        The string/item that you want to print

    Returns
    -------
    None
        No return value
    """
    return Oasys.THIS._connection.functionCall("Print", string)


def Println(string):
    """
    Print a string to stdout adding a carriage return

    Parameters
    ----------
    string : Any valid javascript type
        The string/item that you want to print

    Returns
    -------
    None
        No return value
    """
    return Oasys.THIS._connection.functionCall("Println", string)


def ReturnFlag(flag):
    """
    Return a flag used in the script. See also
    AllocateFlag() and

    Parameters
    ----------
    flag : Flag
        The flag to return

    Returns
    -------
    None
        No return value
    """
    return Oasys.THIS._connection.functionCall("ReturnFlag", flag)


def SetCurrentDirectory(directory_path):
    """
    Sets the current working directory

    Parameters
    ----------
    directory_path : string
        Path to the directory you would like to change into

    Returns
    -------
    bool
        True if successful, False if not
    """
    return Oasys.THIS._connection.functionCall("SetCurrentDirectory", directory_path)


def SetFtcfVar(name):
    """
    Set the value of a FAST-TCF variable. If the variable already exists then it's value is updated

    Parameters
    ----------
    name : string
        The FAST-TCF variable name (case independent)

    Returns
    -------
    str
        String containing variable value or None if variable does not exist
    """
    return Oasys.THIS._connection.functionCall("SetFtcfVar", name)


def Sleep(time):
    """
    Pause execution of the script for time seconds. See also
    MilliSleep()

    Parameters
    ----------
    time : integer
        Number of seconds to pause for

    Returns
    -------
    None
        No return value
    """
    return Oasys.THIS._connection.functionCall("Sleep", time)


def System(string):
    """
    Do a system command outside T/HIS. To run an external command and get the output then please use
    Execute() instead

    Parameters
    ----------
    string : Any valid javascript type
        The system command that you want to do

    Returns
    -------
    int
        integer (probably zero if command successful but is implementation-dependant)
    """
    return Oasys.THIS._connection.functionCall("System", string)


def Unix():
    """
    Test whether script is running on a Unix/Linux operating system. See also
    Windows()

    Returns
    -------
    bool
        True if Unix/Linux, False if not
    """
    return Oasys.THIS._connection.functionCall("Unix")


def UpdateCurveMenu():
    """
    Updates the scroll bar in the Curve Manager. Useful if your script has created lots of new curves and you want to update the menu (otherwise, the scroll bar range will only be updated when the script exits)

    Returns
    -------
    None
        No return value
    """
    return Oasys.THIS._connection.functionCall("UpdateCurveMenu")


def WarningMessage(string):
    """
    Print a warning message to the dialogue box adding a carriage return

    Parameters
    ----------
    string : Any valid javascript type
        The string/item that you want to print

    Returns
    -------
    None
        No return value
    """
    return Oasys.THIS._connection.functionCall("WarningMessage", string)


def Windows():
    """
    Test whether script is running on a Windows operating system. See also
    Unix()

    Returns
    -------
    bool
        True if Windows, False if not
    """
    return Oasys.THIS._connection.functionCall("Windows")



import Oasys.gRPC


# Metaclass for static properties and constants
class LineWidthType(type):
    _consts = {'BOLD', 'FINE', 'HEAVY', 'NORMAL', 'W1', 'W10', 'W2', 'W3', 'W4', 'W5', 'W6', 'W7', 'W8', 'W9'}

    def __getattr__(cls, name):
        if name in LineWidthType._consts:
            return Oasys.THIS._connection.classGetter(cls.__name__, name)

        raise AttributeError


class LineWidth(Oasys.gRPC.OasysItem, metaclass=LineWidthType):


    def __del__(self):
        if not Oasys.THIS._connection:
            return

        Oasys.THIS._connection.destructor(self.__class__.__name__, self._handle)


    def __getattr__(self, name):
        raise AttributeError


    def __setattr__(self, name, value):
# Set the property locally
        self.__dict__[name] = value



import Oasys.gRPC


# Metaclass for static properties and constants
class GroupType(type):

    def __getattr__(cls, name):

        raise AttributeError


class Group(Oasys.gRPC.OasysItem, metaclass=GroupType):
    _props = {'crv_at_ymax', 'crv_at_ymin', 'curves', 'name', 'x_at_ymax', 'x_at_ymin', 'x_at_yminpos', 'xmax', 'xmin', 'xminpos', 'ymax', 'ymin', 'yminpos'}


    def __del__(self):
        if not Oasys.THIS._connection:
            return

        Oasys.THIS._connection.destructor(self.__class__.__name__, self._handle)


    def __getattr__(self, name):
# If one of the properties we define then get it
        if name in Group._props:
            return Oasys.THIS._connection.instanceGetter(self.__class__.__name__, self._handle, name)

        raise AttributeError


    def __setattr__(self, name, value):
# If one of the properties we define then set it
        if name in Group._props:
            Oasys.THIS._connection.instanceSetter(self.__class__.__name__, self._handle, name, value)
            return

# Set the property locally
        self.__dict__[name] = value


# Constructor
    def __init__(self, name):
        handle = Oasys.THIS._connection.constructor(self.__class__.__name__, name)
        Oasys.gRPC.OasysItem.__init__(self, self.__class__.__name__, handle)
        """
        Create a new Group object

        Parameters
        ----------
        name : string
            Group name used to reference the group

        Returns
        -------
        Group
            Group object
        """


# Static methods
    def DeleteGroup(group_id_or_name, delete_automatic_groups=Oasys.gRPC.defaultArg):
        """
        Deletes a curve group

        Parameters
        ----------
        group_id_or_name : integer or string
            ID of group to delete or name of group. If this argument is 0, delete all groups. Automatically generated groups won't be deleted unless the next argument is set to 1
        delete_automatic_groups : integer
            Optional. If this argument is 1, automatic groups can be deleted. If no argument or 0, automatic groups cant be deleted

        Returns
        -------
        None
            No return value
        """
        return Oasys.THIS._connection.classMethod(__class__.__name__, "DeleteGroup", group_id_or_name, delete_automatic_groups)

    def Get(name):
        """
        Returns a group object

        Parameters
        ----------
        name : string
            Name of the group to return object for

        Returns
        -------
        Group
            Group object (or None if the group does not exist)
        """
        return Oasys.THIS._connection.classMethod(__class__.__name__, "Get", name)

    def GetFromID(id):
        """
        Returns a group object

        Parameters
        ----------
        id : integer
            ID of the group to return object for

        Returns
        -------
        Group
            Group object (or None if the group does not exist)
        """
        return Oasys.THIS._connection.classMethod(__class__.__name__, "GetFromID", id)

    def Total():
        """
        Returns the total number of curve group currently defined

        Returns
        -------
        int
            Number of curve groups currently defined
        """
        return Oasys.THIS._connection.classMethod(__class__.__name__, "Total")



# Instance methods
    def Add(self, curve):
        """
        Adds a curve object to group

        Parameters
        ----------
        curve : Curve
            Curve that will be added to group

        Returns
        -------
        None
            No return value
        """
        return Oasys.THIS._connection.instanceMethod(self.__class__.__name__, self._handle, "Add", curve)

    def AddAll(self):
        """
        Adds all curves to group

        Returns
        -------
        None
            No return value
        """
        return Oasys.THIS._connection.instanceMethod(self.__class__.__name__, self._handle, "AddAll")

    def AddID(self, id):
        """
        Adds curve by ID to a group

        Parameters
        ----------
        id : integer
            The ID of the curve you want to add

        Returns
        -------
        None
            No return value
        """
        return Oasys.THIS._connection.instanceMethod(self.__class__.__name__, self._handle, "AddID", id)

    def Contains(self, curve):
        """
        Checks if a curve object is in a curve group

        Parameters
        ----------
        curve : Curve
            Curve that will be checked

        Returns
        -------
        bool
            True if the curve is in the group, otherwise False
        """
        return Oasys.THIS._connection.instanceMethod(self.__class__.__name__, self._handle, "Contains", curve)

    def ContainsID(self, id):
        """
        Checks if a curve ID is in a curve group

        Parameters
        ----------
        id : integer
            The ID of the curve you want to check

        Returns
        -------
        bool
            True if the curve is in the group, otherwise False
        """
        return Oasys.THIS._connection.instanceMethod(self.__class__.__name__, self._handle, "ContainsID", id)

    def GetCurveIDs(self):
        """
        Returns a list of Curve ID's for all the Curves in the group

        Returns
        -------
        int
            List of integers
        """
        return Oasys.THIS._connection.instanceMethod(self.__class__.__name__, self._handle, "GetCurveIDs")

    def GetCurves(self):
        """
        Returns a list of Curve Objects for all the Curves in the group

        Returns
        -------
        list
            List of Curve objects
        """
        return Oasys.THIS._connection.instanceMethod(self.__class__.__name__, self._handle, "GetCurves")

    def Remove(self, curve):
        """
        Removes a curve object from a group

        Parameters
        ----------
        curve : Curve
            Curve that will be removed from group

        Returns
        -------
        None
            No return value
        """
        return Oasys.THIS._connection.instanceMethod(self.__class__.__name__, self._handle, "Remove", curve)

    def RemoveAll(self):
        """
        Removes all curves from a group

        Returns
        -------
        None
            No return value
        """
        return Oasys.THIS._connection.instanceMethod(self.__class__.__name__, self._handle, "RemoveAll")

    def RemoveID(self, id):
        """
        Remove a curve by ID from a group

        Parameters
        ----------
        id : integer
            The ID of the curve you want to remove

        Returns
        -------
        None
            No return value
        """
        return Oasys.THIS._connection.instanceMethod(self.__class__.__name__, self._handle, "RemoveID", id)

    def Spool(self):
        """
        Spools a group, entry by entry and returns the curve objects. See also Group.StartSpool

        Returns
        -------
        Curve
            Curve Object of item, or None if no more curves in group
        """
        return Oasys.THIS._connection.instanceMethod(self.__class__.__name__, self._handle, "Spool")

    def SpoolID(self):
        """
        Spools a group, entry by entry and returns the curve ID's or 0 when no more curves in group. See also Group.StartSpool

        Returns
        -------
        int
            integer
        """
        return Oasys.THIS._connection.instanceMethod(self.__class__.__name__, self._handle, "SpoolID")

    def StartSpool(self):
        """
        Starts a group spooling operation. See also Group.Spool

        Returns
        -------
        None
            No return value
        """
        return Oasys.THIS._connection.instanceMethod(self.__class__.__name__, self._handle, "StartSpool")




import Oasys.gRPC


# Metaclass for static properties and constants
class CurveType(type):
    _consts = {'AFTER', 'BEFORE'}

    def __getattr__(cls, name):
        if name in CurveType._consts:
            return Oasys.THIS._connection.classGetter(cls.__name__, name)

        raise AttributeError


class Curve(Oasys.gRPC.OasysItem, metaclass=CurveType):
    _props = {'average', 'colour', 'directory', 'entity_id', 'entity_type', 'file', 'hic', 'hic_tmax', 'hic_tmin', 'hicd', 'hicd_tmax', 'hicd_tmin', 'id', 'is_null', 'label', 'model', 'npoints', 'regr_rsq', 'regr_sdgrad', 'regr_sdicpt', 'regr_sdyx', 'rms', 'style', 'symbol', 'tag', 'title', 'tms', 'tms_tmax', 'tms_tmin', 'unit_system', 'width', 'x_at_ymax', 'x_at_ymin', 'x_axis_label', 'x_axis_unit', 'xmax', 'xmin', 'y_axis_label', 'y_axis_unit', 'ymax', 'ymin'}


    def __del__(self):
        if not Oasys.THIS._connection:
            return

        Oasys.THIS._connection.destructor(self.__class__.__name__, self._handle)


    def __getattr__(self, name):
# If one of the properties we define then get it
        if name in Curve._props:
            return Oasys.THIS._connection.instanceGetter(self.__class__.__name__, self._handle, name)

        raise AttributeError


    def __setattr__(self, name, value):
# If one of the properties we define then set it
        if name in Curve._props:
            Oasys.THIS._connection.instanceSetter(self.__class__.__name__, self._handle, name, value)
            return

# Set the property locally
        self.__dict__[name] = value


# Constructor
    def __init__(self, lcid, tag=Oasys.gRPC.defaultArg, line_label=Oasys.gRPC.defaultArg, x_axis_label=Oasys.gRPC.defaultArg, y_axis_label=Oasys.gRPC.defaultArg):
        handle = Oasys.THIS._connection.constructor(self.__class__.__name__, lcid, tag, line_label, x_axis_label, y_axis_label)
        Oasys.gRPC.OasysItem.__init__(self, self.__class__.__name__, handle)
        """
        Create a new Curve object. 
        The curve will be added to all the currently active graphs

        Parameters
        ----------
        lcid : integer
            Curve number
        tag : string
            Optional. Tag used to reference the curve in FAST-TCF scripts
        line_label : string
            Optional. Line label for the curve
        x_axis_label : string
            Optional. X-axis label for the curve
        y_axis_label : string
            Optional. Y-axis label for the curve

        Returns
        -------
        Curve
            Curve object
        """


# Static methods
    def AddFlaggedToGraph(flag, *graph):
        """
        Adds flagged curves to a graph

        Parameters
        ----------
        flag : Flag
            Flag to check on the curve
        *graph : int
            Graph to add the curve to. If undefined then the curve is added to all graphs
            This argument can be repeated if required.
            Alternatively a single array argument containing the multiple values can be given

        Returns
        -------
        None
            No return value
        """
        return Oasys.THIS._connection.classMethod(__class__.__name__, "AddFlaggedToGraph", flag, graph)

    def Copy(source, target):
        """
        Copies a curve

        Parameters
        ----------
        source : integer
            ID of curve to copy from
        target : integer
            ID of curve to copy to

        Returns
        -------
        None
            No return value
        """
        return Oasys.THIS._connection.classMethod(__class__.__name__, "Copy", source, target)

    def Delete(curve):
        """
        Deletes a curve

        Parameters
        ----------
        curve : integer
            ID of curve to delete

        Returns
        -------
        None
            No return value
        """
        return Oasys.THIS._connection.classMethod(__class__.__name__, "Delete", curve)

    def DeleteFlagged(flag):
        """
        Deletes flagged curves

        Parameters
        ----------
        flag : Flag
            Flag to check on the curve

        Returns
        -------
        None
            No return value
        """
        return Oasys.THIS._connection.classMethod(__class__.__name__, "DeleteFlagged", flag)

    def Exists(curve):
        """
        Checks if a curve exists

        Parameters
        ----------
        curve : integer
            ID of curve to check

        Returns
        -------
        bool
            True if the curve exists, otherwise False
        """
        return Oasys.THIS._connection.classMethod(__class__.__name__, "Exists", curve)

    def First():
        """
        Returns the first curve

        Returns
        -------
        Curve
            Curve object (or None if there are no more curves in the model)
        """
        return Oasys.THIS._connection.classMethod(__class__.__name__, "First")

    def FirstFreeID():
        """
        Returns the ID of the first free curve

        Returns
        -------
        int
            ID of first unsued curve
        """
        return Oasys.THIS._connection.classMethod(__class__.__name__, "FirstFreeID")

    def FirstID():
        """
        Returns the ID of the first curve

        Returns
        -------
        int
            ID of the first curve defined
        """
        return Oasys.THIS._connection.classMethod(__class__.__name__, "FirstID")

    def FlagAll(flag):
        """
        Flags all of the curves with a defined flag

        Parameters
        ----------
        flag : integer
            Flag to set on the curves

        Returns
        -------
        None
            No return value
        """
        return Oasys.THIS._connection.classMethod(__class__.__name__, "FlagAll", flag)

    def GetFlagged(flag):
        """
        Returns a list of all curves flagged with a given flag

        Parameters
        ----------
        flag : Flag
            Flag for which to return flagged objects

        Returns
        -------
        list
            List of Curve objects (or None if no curves are flagged)
        """
        return Oasys.THIS._connection.classMethod(__class__.__name__, "GetFlagged", flag)

    def GetFromID(id):
        """
        Returns the curve object for a curve ID

        Parameters
        ----------
        id : integer
            ID of curve to return object for

        Returns
        -------
        Curve
            Curve object (or None if the curve does not exist
        """
        return Oasys.THIS._connection.classMethod(__class__.__name__, "GetFromID", id)

    def GetFromTag(tag):
        """
        Finds a curve from it's Tag. This function is only available when running a Javascript from within a FAST-TCF script

        Parameters
        ----------
        tag : string
            TAG of curve to return object for

        Returns
        -------
        Curve
            Curve object (or None if there are no free curves)
        """
        return Oasys.THIS._connection.classMethod(__class__.__name__, "GetFromTag", tag)

    def HighestID():
        """
        Returns the ID of the highest curve currently being used

        Returns
        -------
        int
            ID of highest curve currently being used
        """
        return Oasys.THIS._connection.classMethod(__class__.__name__, "HighestID")

    def Pick(prompt, modal=Oasys.gRPC.defaultArg):
        """
        Picks a single curve

        Parameters
        ----------
        prompt : string
            Text to display as a prompt to the user
        modal : boolean
            Optional. If selection is modal (blocks the user from doing anything else in T/HIS
            until this window is dismissed). If omitted the selection will be modal

        Returns
        -------
        Curve
            Curve object (or None if the user cancels the pick operation)
        """
        return Oasys.THIS._connection.classMethod(__class__.__name__, "Pick", prompt, modal)

    def RemoveFlaggedFromGraph(flag, *graph):
        """
        Removes flagged curves from a graph

        Parameters
        ----------
        flag : Flag
            Flag to check on the curve
        *graph : int
            Graph to remove the curve from. If undefined then the curve is removed from all graphs
            This argument can be repeated if required.
            Alternatively a single array argument containing the multiple values can be given

        Returns
        -------
        None
            No return value
        """
        return Oasys.THIS._connection.classMethod(__class__.__name__, "RemoveFlaggedFromGraph", flag, graph)

    def Select(flag, prompt, modal=Oasys.gRPC.defaultArg):
        """
        Allows the user to select curves

        Parameters
        ----------
        flag : integer
            Flag to use when selecting curves
        prompt : string
            Text to display as a prompt to the user
        modal : boolean
            Optional. If selection is modal (blocks the user from doing anything else in T/HIS
            until this window is dismissed). If omitted the selection will be modal

        Returns
        -------
        int
            Number of items selected or None if menu cancelled
        """
        return Oasys.THIS._connection.classMethod(__class__.__name__, "Select", flag, prompt, modal)

    def UnflagAll(flag):
        """
        Unsets a defined flag on all of the curves

        Parameters
        ----------
        flag : integer
            Flag to unset on the curves

        Returns
        -------
        None
            No return value
        """
        return Oasys.THIS._connection.classMethod(__class__.__name__, "UnflagAll", flag)



# Instance methods
    def AddPoint(self, xvalue, yvalue):
        """
        Adds a point at the end of the curve

        Parameters
        ----------
        xvalue : real
            The x value of the point
        yvalue : real
            The y value of the point

        Returns
        -------
        None
            No return value
        """
        return Oasys.THIS._connection.instanceMethod(self.__class__.__name__, self._handle, "AddPoint", xvalue, yvalue)

    def AddToGraph(self, *graph):
        """
        Adds a curve to a graph

        Parameters
        ----------
        *graph : int
            Graph to add the curve to. If undefined then the curve is added to all graphs
            This argument can be repeated if required.
            Alternatively a single array argument containing the multiple values can be given

        Returns
        -------
        None
            No return value
        """
        return Oasys.THIS._connection.instanceMethod(self.__class__.__name__, self._handle, "AddToGraph", graph)

    def ClearFlag(self, flag):
        """
        Clears a flag on the curve

        Parameters
        ----------
        flag : integer
            Flag to clear on the curve

        Returns
        -------
        None
            No return value
        """
        return Oasys.THIS._connection.instanceMethod(self.__class__.__name__, self._handle, "ClearFlag", flag)

    def DeletePoint(self, ipt):
        """
        Deletes a point in a curve. The input
        for the point number should start at 1 for the 1st point not zero

        Parameters
        ----------
        ipt : integer
            The point you want to insert the data before or after

        Returns
        -------
        None
            No return value
        """
        return Oasys.THIS._connection.instanceMethod(self.__class__.__name__, self._handle, "DeletePoint", ipt)

    def Flagged(self, flag):
        """
        Checks if the curve is flagged or not

        Parameters
        ----------
        flag : integer
            Flag to check on the curve

        Returns
        -------
        bool
            True if flagged, False if not
        """
        return Oasys.THIS._connection.instanceMethod(self.__class__.__name__, self._handle, "Flagged", flag)

    def Freeze(self, graph, freeze_option):
        """
        Freezes an unblanked curve on one or all graphs

        Parameters
        ----------
        graph : integer
            Graph number to freeze curve on or 0 for all graphs
        freeze_option : integer
            No argument or 1 to freeze the curve, 0 to unfreeze

        Returns
        -------
        None
            No return value
        """
        return Oasys.THIS._connection.instanceMethod(self.__class__.__name__, self._handle, "Freeze", graph, freeze_option)

    def GetPoint(self, row):
        """
        Returns x and y data for a point in a curve. The input
        for the point number should start at 1 for the 1st point not zero. In the list
        returned array[0] contains the x axis value and array[1] contains the y-axis value

        Parameters
        ----------
        row : integer
            The point you want the data for

        Returns
        -------
        list
            Array of point values
        """
        return Oasys.THIS._connection.instanceMethod(self.__class__.__name__, self._handle, "GetPoint", row)

    def InsertPoint(self, ipt, xvalue, yvalue, position):
        """
        Inserts a new point before or after the specified point

        Parameters
        ----------
        ipt : integer
            The point you want to insert the data before or after
        xvalue : real
            The x value of the point
        yvalue : real
            The y value of the point
        position : integer
            Specify either before or after the selected pioint. Use 'Curve.BEFORE' for before, and 'Curve.AFTER' for after

        Returns
        -------
        None
            No return value
        """
        return Oasys.THIS._connection.instanceMethod(self.__class__.__name__, self._handle, "InsertPoint", ipt, xvalue, yvalue, position)

    def Next(self):
        """
        Returns the next curve in the model

        Returns
        -------
        Curve
            Curve object (or None if there are no more curves in the model)
        """
        return Oasys.THIS._connection.instanceMethod(self.__class__.__name__, self._handle, "Next")

    def Previous(self):
        """
        Returns the previous curve in the model

        Returns
        -------
        Curve
            Curve object (or None if there are no more curves in the model)
        """
        return Oasys.THIS._connection.instanceMethod(self.__class__.__name__, self._handle, "Previous")

    def RemoveFromGraph(self, *graph):
        """
        Removes a curve from a graph

        Parameters
        ----------
        *graph : int
            Graph to remove the curve from, If undefined then the curve is removed from all graphs
            This argument can be repeated if required.
            Alternatively a single array argument containing the multiple values can be given

        Returns
        -------
        None
            No return value
        """
        return Oasys.THIS._connection.instanceMethod(self.__class__.__name__, self._handle, "RemoveFromGraph", graph)

    def SetFlag(self, flag):
        """
        Sets a flag on the curve

        Parameters
        ----------
        flag : integer
            Flag to set on the curve

        Returns
        -------
        None
            No return value
        """
        return Oasys.THIS._connection.instanceMethod(self.__class__.__name__, self._handle, "SetFlag", flag)

    def SetPoint(self, ipt, xvalue, yvalue):
        """
        Sets the x and y values for a specified point in a curve

        Parameters
        ----------
        ipt : integer
            The point to set the data for
        xvalue : real
            The x value of the point
        yvalue : real
            The y value of the point

        Returns
        -------
        None
            No return value
        """
        return Oasys.THIS._connection.instanceMethod(self.__class__.__name__, self._handle, "SetPoint", ipt, xvalue, yvalue)

    def Update(self):
        """
        Updates a curve properties (min,max, average values etc)

        Returns
        -------
        None
            No return value
        """
        return Oasys.THIS._connection.instanceMethod(self.__class__.__name__, self._handle, "Update")

    def YatX(self, xvalue):
        """
        Returns the y value of the curve at a given x value, interpolating if requested x value lies between data points

        Parameters
        ----------
        xvalue : real
            The x value

        Returns
        -------
        float
            Y value
        """
        return Oasys.THIS._connection.instanceMethod(self.__class__.__name__, self._handle, "YatX", xvalue)




import Oasys.gRPC


# Metaclass for static properties and constants
class WindowType(type):
    _consts = {'CANCEL', 'NO', 'NONMODAL', 'OK', 'YES'}

    def __getattr__(cls, name):
        if name in WindowType._consts:
            return Oasys.THIS._connection.classGetter(cls.__name__, name)

        raise AttributeError


class Window(Oasys.gRPC.OasysItem, metaclass=WindowType):


    def __del__(self):
        if not Oasys.THIS._connection:
            return

        Oasys.THIS._connection.destructor(self.__class__.__name__, self._handle)


    def __getattr__(self, name):
        raise AttributeError


    def __setattr__(self, name, value):
# Set the property locally
        self.__dict__[name] = value


# Static methods
    def Error(title, error, buttons=Oasys.gRPC.defaultArg):
        """
        Show an error message in a window

        Parameters
        ----------
        title : string
            Title for window
        error : string
            Error message to show in window.
            The maximum number of lines that can be shown is controlled by the
            Options.max_window_lines option
        buttons : constant
            Optional. The buttons to use. Can be bitwise OR of Window.OK,
            Window.CANCEL,
            Window.YES or
            Window.NO. If this is omitted an OK button will be used.
            By default the window will be modal. If Window.NONMODAL
            is also given the window will be non-modal instead

        Returns
        -------
        int
            Button pressed
        """
        return Oasys.THIS._connection.classMethod(__class__.__name__, "Error", title, error, buttons)

    def GetDirectory(initial=Oasys.gRPC.defaultArg):
        """
        Map the directory selector box native to your machine, allowing you to choose a directory.
        On Unix this will be a Motif selector. Windows will use the standard windows directory selector

        Parameters
        ----------
        initial : string
            Optional. Initial directory to start from

        Returns
        -------
        str
            directory (string), (or None if cancel pressed)
        """
        return Oasys.THIS._connection.classMethod(__class__.__name__, "GetDirectory", initial)

    def GetFile(extension=Oasys.gRPC.defaultArg, save=Oasys.gRPC.defaultArg, initial=Oasys.gRPC.defaultArg):
        """
        Map a file selector box allowing you to choose a file.
        See also Window.GetFiles() and
        Window.GetFilename()

        Parameters
        ----------
        extension : string
            Optional. Extension to filter by
        save : boolean
            Optional. If true the file selector is to be used for saving a file. If false (default) the file selector is for opening
            a file.
            Due to native operating system file selector differences, on linux new filenames can only be given when saving a file.
            On windows it is possible to give new filenames when opening or saving a file
        initial : string
            Optional. Initial directory to start from

        Returns
        -------
        str
            filename (string), (or None if cancel pressed)
        """
        return Oasys.THIS._connection.classMethod(__class__.__name__, "GetFile", extension, save, initial)

    def GetFilename(title, message, extension=Oasys.gRPC.defaultArg, initial=Oasys.gRPC.defaultArg, save=Oasys.gRPC.defaultArg):
        """
        Map a window allowing you to input a filename (or select it using a file selector). OK and Cancel buttons are shown.
        See also Window.GetFile()

        Parameters
        ----------
        title : string
            Title for window
        message : string
            Message to show in window
        extension : string
            Optional. Extension to filter by
        initial : string
            Optional. Initial value
        save : boolean
            Optional. If true the file selector is to be used for saving a file. If false (default) the file selector is for opening
            a file.
            Due to native operating system file selector differences, on linux new filenames can only be given when saving a file.
            On windows it is possible to give new filenames when opening or saving a file

        Returns
        -------
        str
            filename (string), (or None if cancel pressed)
        """
        return Oasys.THIS._connection.classMethod(__class__.__name__, "GetFilename", title, message, extension, initial, save)

    def GetFiles(extension=Oasys.gRPC.defaultArg):
        """
        Map a file selector box allowing you to choose multiple files.
        See also Window.GetFile() and
        Window.GetFilename()

        Parameters
        ----------
        extension : string
            Optional. Extension to filter by

        Returns
        -------
        str
            List of filenames (strings), or None if cancel pressed
        """
        return Oasys.THIS._connection.classMethod(__class__.__name__, "GetFiles", extension)

    def GetInteger(title, message, initial=Oasys.gRPC.defaultArg):
        """
        Map a window allowing you to input an integer. OK and Cancel buttons are shown

        Parameters
        ----------
        title : string
            Title for window
        message : string
            Message to show in window
        initial : integer
            Optional. Initial value

        Returns
        -------
        int
            value input (integer), or None if cancel pressed
        """
        return Oasys.THIS._connection.classMethod(__class__.__name__, "GetInteger", title, message, initial)

    def GetNumber(title, message, initial=Oasys.gRPC.defaultArg):
        """
        Map a window allowing you to input a number. OK and Cancel buttons are shown

        Parameters
        ----------
        title : string
            Title for window
        message : string
            Message to show in window
        initial : float
            Optional. Initial value

        Returns
        -------
        float
            value input (float), or None if cancel pressed
        """
        return Oasys.THIS._connection.classMethod(__class__.__name__, "GetNumber", title, message, initial)

    def GetPassword(title, message):
        """
        Map a window allowing you to input a password. OK and Cancel buttons are shown.
        This is identical to Window.GetString except the
        string is hidden and no initial value can be given

        Parameters
        ----------
        title : string
            Title for window
        message : string
            Message to show in window

        Returns
        -------
        str
            value input (string), or None if cancel pressed
        """
        return Oasys.THIS._connection.classMethod(__class__.__name__, "GetPassword", title, message)

    def GetString(title, message, initial=Oasys.gRPC.defaultArg):
        """
        Map a window allowing you to input a string. OK and Cancel buttons are shown

        Parameters
        ----------
        title : string
            Title for window
        message : string
            Message to show in window
        initial : string
            Optional. Initial value

        Returns
        -------
        str
            value input (string), or None if cancel pressed
        """
        return Oasys.THIS._connection.classMethod(__class__.__name__, "GetString", title, message, initial)

    def Information(title, info, buttons=Oasys.gRPC.defaultArg):
        """
        Show information in a window

        Parameters
        ----------
        title : string
            Title for window
        info : string
            Information to show in window.
            The maximum number of lines that can be shown is controlled by the
            Options.max_window_lines option
        buttons : constant
            Optional. The buttons to use. Can be bitwise OR of Window.OK,
            Window.CANCEL,
            Window.YES or
            Window.NO. If this is omitted an OK button will be used.
            By default the window will be modal. If Window.NONMODAL
            is also given the window will be non-modal instead

        Returns
        -------
        int
            Button pressed
        """
        return Oasys.THIS._connection.classMethod(__class__.__name__, "Information", title, info, buttons)

    def Message(title, message, buttons=Oasys.gRPC.defaultArg):
        """
        Show a message in a window

        Parameters
        ----------
        title : string
            Title for window
        message : string
            Message to show in window.
            The maximum number of lines that can be shown is controlled by the
            Options.max_window_lines option
        buttons : constant
            Optional. The buttons to use. Can be bitwise OR of Window.OK,
            Window.CANCEL,
            Window.YES or
            Window.NO. If this is omitted an OK button will be used
            By default the window will be modal. If Window.NONMODAL
            is also given the window will be non-modal instead

        Returns
        -------
        int
            Button pressed
        """
        return Oasys.THIS._connection.classMethod(__class__.__name__, "Message", title, message, buttons)

    def Question(title, question, buttons=Oasys.gRPC.defaultArg):
        """
        Show a question in a window

        Parameters
        ----------
        title : string
            Title for window
        question : string
            Question to show in window.
            The maximum number of lines that can be shown is controlled by the
            Options.max_window_lines option
        buttons : constant
            Optional. The buttons to use. Can be bitwise OR of Window.OK,
            Window.CANCEL,
            Window.YES or
            Window.NO. If this is omitted Yes and No button will be used.
            By default the window will be modal. If Window.NONMODAL
            is also given the window will be non-modal instead

        Returns
        -------
        int
            Button pressed
        """
        return Oasys.THIS._connection.classMethod(__class__.__name__, "Question", title, question, buttons)

    def Warning(title, warning, buttons=Oasys.gRPC.defaultArg):
        """
        Show a warning message in a window

        Parameters
        ----------
        title : string
            Title for window
        warning : string
            Warning message to show in window.
            The maximum number of lines that can be shown is controlled by the
            Options.max_window_lines option
        buttons : constant
            Optional. The buttons to use. Can be bitwise OR of Window.OK,
            Window.CANCEL,
            Window.YES or
            Window.NO. If this is omitted an OK button will be used.
            By default the window will be modal. If Window.NONMODAL
            is also given the window will be non-modal instead

        Returns
        -------
        int
            Button pressed
        """
        return Oasys.THIS._connection.classMethod(__class__.__name__, "Warning", title, warning, buttons)




from .colour                         import Colour
from .component                      import Component
from .curve                          import Curve
from .datum                          import Datum
from .entity                         import Entity
from .graph                          import Graph
from .group                          import Group
from .include                        import Include
from .linestyle                      import LineStyle
from .linewidth                      import LineWidth
from .model                          import Model
from .operate                        import Operate
from .options                        import Options
from .page                           import Page
from .read                           import Read
from .symbol                         import Symbol
from .unitsystem                     import UnitSystem
from .units                          import Units
from .utils                          import Utils
from .window                         import Window
from .workflow                       import Workflow

from .itemobject                     import ItemObject
from .funcs                          import *
from .core                           import *



import Oasys.gRPC


# Metaclass for static properties and constants
class OperateType(type):

    def __getattr__(cls, name):

        raise AttributeError


class Operate(Oasys.gRPC.OasysItem, metaclass=OperateType):


    def __del__(self):
        if not Oasys.THIS._connection:
            return

        Oasys.THIS._connection.destructor(self.__class__.__name__, self._handle)


    def __getattr__(self, name):
        raise AttributeError


    def __setattr__(self, name, value):
# Set the property locally
        self.__dict__[name] = value


# Static methods
    def Abs(input_curve, output_curve=Oasys.gRPC.defaultArg):
        """
        Convert a curve to absolute values

        Parameters
        ----------
        input_curve : Curve
            Input Curve
        output_curve : Curve
            Optional. Curve to overwrite

        Returns
        -------
        Curve
            Curve object or None
        """
        return Oasys.THIS._connection.classMethod(__class__.__name__, "Abs", input_curve, output_curve)

    def Acos(input_curve, output_curve=Oasys.gRPC.defaultArg):
        """
        Calculate Arc Cosine

        Parameters
        ----------
        input_curve : Curve
            Input Curve
        output_curve : Curve
            Optional. Curve to overwrite

        Returns
        -------
        Curve
            Curve object or None
        """
        return Oasys.THIS._connection.classMethod(__class__.__name__, "Acos", input_curve, output_curve)

    def Acu(input_curve, offset, time_period, output_curve=Oasys.gRPC.defaultArg):
        """
        Evaluates the integratal of a curve over a user defined period

        Parameters
        ----------
        input_curve : Curve
            Input Curve
        offset : float
            User defined offset
        time_period : float
            Time to integrate over
        output_curve : Curve
            Optional. Curve to overwrite

        Returns
        -------
        Curve
            Curve object or None
        """
        return Oasys.THIS._connection.classMethod(__class__.__name__, "Acu", input_curve, offset, time_period, output_curve)

    def Ad(input_curve, output_curve=Oasys.gRPC.defaultArg):
        """
        Convert acceleration spectrum to a displacment spectrum

        Parameters
        ----------
        input_curve : Curve
            Input Curve
        output_curve : Curve
            Optional. Curve to overwrite

        Returns
        -------
        Curve
            Curve object or None
        """
        return Oasys.THIS._connection.classMethod(__class__.__name__, "Ad", input_curve, output_curve)

    def Add(input_curve, second_curve_or_constant, output_curve=Oasys.gRPC.defaultArg):
        """
        Add Y axis values

        Parameters
        ----------
        input_curve : Curve
            Input Curve
        second_curve_or_constant : Curve or real
            Second Curve or constant
        output_curve : Curve
            Optional. Curve to overwrite

        Returns
        -------
        Curve
            Curve object or None
        """
        return Oasys.THIS._connection.classMethod(__class__.__name__, "Add", input_curve, second_curve_or_constant, output_curve)

    def Adx(first_curve, second_curve_or_constant, output_curve=Oasys.gRPC.defaultArg):
        """
        Add X axis values

        Parameters
        ----------
        first_curve : Curve
            First Curve
        second_curve_or_constant : Curve or real
            Second Curve or constant
        output_curve : Curve
            Optional. Curve to overwrite

        Returns
        -------
        Curve
            Curve object or None
        """
        return Oasys.THIS._connection.classMethod(__class__.__name__, "Adx", first_curve, second_curve_or_constant, output_curve)

    def Asi(x_acceleration, y_acceleration, z_acceleration, acceleration_conversion_factor, x_acceleration_limit, y_acceleration_limit, z_acceleration_limit, calculation_method, x_axis_interval=Oasys.gRPC.defaultArg, output_curve=Oasys.gRPC.defaultArg):
        """
        Acceleration Severity Index. This value is used to assess the performance of road side
        crash barriers. The calculation method can be set to 2010 (BS EN 1317-1:2010) or 1998 (BS EN 1317-1:1998)

        Parameters
        ----------
        x_acceleration : Curve
            X Acceleration Curve
        y_acceleration : Curve
            Y Acceleration Curve
        z_acceleration : Curve
            Z Acceleration Curve
        acceleration_conversion_factor : float
            Factor required to divide input acceleration curve by to convert to (G)
        x_acceleration_limit : float
            X direction acceleration limit
        y_acceleration_limit : float
            Y direction acceleration limit
        z_acceleration_limit : float
            Z direction acceleration limit
        calculation_method : string
            Either 2010 or 1998
        x_axis_interval : float
            Optional. If defined then T-HIS will automatically regularise the curve using this value first
        output_curve : Curve
            Optional. Curve to overwrite

        Returns
        -------
        Curve
            Curve object or None
        """
        return Oasys.THIS._connection.classMethod(__class__.__name__, "Asi", x_acceleration, y_acceleration, z_acceleration, acceleration_conversion_factor, x_acceleration_limit, y_acceleration_limit, z_acceleration_limit, calculation_method, x_axis_interval, output_curve)

    def Asin(input_curve, output_curve=Oasys.gRPC.defaultArg):
        """
        Calculate Arc Sine

        Parameters
        ----------
        input_curve : Curve
            Input Curve
        output_curve : Curve
            Optional. Curve to overwrite

        Returns
        -------
        Curve
            Curve object or None
        """
        return Oasys.THIS._connection.classMethod(__class__.__name__, "Asin", input_curve, output_curve)

    def Atan(input_curve, output_curve=Oasys.gRPC.defaultArg):
        """
        Calculate Arc Tangent

        Parameters
        ----------
        input_curve : Curve
            Input Curve
        output_curve : Curve
            Optional. Curve to overwrite

        Returns
        -------
        Curve
            Curve object or None
        """
        return Oasys.THIS._connection.classMethod(__class__.__name__, "Atan", input_curve, output_curve)

    def Atan2(first_input_curve, second_input_curve, output_curve=Oasys.gRPC.defaultArg):
        """
        Calculate Arc Tangent using atan2(y, x)

        Parameters
        ----------
        first_input_curve : Curve
            Input Curve
        second_input_curve : Curve
            Input Curve
        output_curve : Curve
            Optional. Curve to overwrite

        Returns
        -------
        Curve
            Curve object or None
        """
        return Oasys.THIS._connection.classMethod(__class__.__name__, "Atan2", first_input_curve, second_input_curve, output_curve)

    def Av(input_curve, output_curve=Oasys.gRPC.defaultArg):
        """
        Convert acceleration spectrum to a velocity spectrum

        Parameters
        ----------
        input_curve : Curve
            Input Curve
        output_curve : Curve
            Optional. Curve to overwrite

        Returns
        -------
        Curve
            Curve object or None
        """
        return Oasys.THIS._connection.classMethod(__class__.__name__, "Av", input_curve, output_curve)

    def Ave(curves, output_curve=Oasys.gRPC.defaultArg):
        """
        Average a group of curves

        Parameters
        ----------
        curves : List of Curve objects
            List of Curve objects
        output_curve : Curve
            Optional. Curve to overwrite

        Returns
        -------
        Curve
            Curve object or None
        """
        return Oasys.THIS._connection.classMethod(__class__.__name__, "Ave", curves, output_curve)

    def Bes(input_curve, frequency, order, x_axis_interval=Oasys.gRPC.defaultArg, output_curve=Oasys.gRPC.defaultArg):
        """
        Bessel Filter

        Parameters
        ----------
        input_curve : Curve
            Input Curve
        frequency : float
            Cut-off Frequency (Hz)
        order : integer
            Filter order
        x_axis_interval : float
            Optional. If defined then T-HIS will automatically regularise the curve using this value first
        output_curve : Curve
            Optional. Curve to overwrite

        Returns
        -------
        Curve
            Curve object or None
        """
        return Oasys.THIS._connection.classMethod(__class__.__name__, "Bes", input_curve, frequency, order, x_axis_interval, output_curve)

    def Blc(input_curve):
        """
        Carry out a baseline correction on an accleration time history

        Parameters
        ----------
        input_curve : Curve
            Moment / Time Curve

        Returns
        -------
        list
            List of Curve objects.
            1st curve : Corrected curve
            2nd curve : Integrated Velocity
            3rd curve : Integrated Displacement
        """
        return Oasys.THIS._connection.classMethod(__class__.__name__, "Blc", input_curve)

    def But(input_curve, frequency, order, x_axis_interval=Oasys.gRPC.defaultArg, output_curve=Oasys.gRPC.defaultArg):
        """
        Butterworth Filter

        Parameters
        ----------
        input_curve : Curve
            Input Curve
        frequency : float
            Cut-off Frequency (Hz)
        order : integer
            Filter order
        x_axis_interval : float
            Optional. If defined then T-HIS will automatically regularise the curve using this value first
        output_curve : Curve
            Optional. Curve to overwrite

        Returns
        -------
        Curve
            Curve object or None
        """
        return Oasys.THIS._connection.classMethod(__class__.__name__, "But", input_curve, frequency, order, x_axis_interval, output_curve)

    def C1000(input_curve, x_axis_interval=Oasys.gRPC.defaultArg, output_curve=Oasys.gRPC.defaultArg):
        """
        SAE Class 1000 Filter

        Parameters
        ----------
        input_curve : Curve
            Input Curve
        x_axis_interval : float
            Optional. If defined then T-HIS will automatically regularise the curve using this value first
        output_curve : Curve
            Optional. Curve to overwrite

        Returns
        -------
        Curve
            Curve object or None
        """
        return Oasys.THIS._connection.classMethod(__class__.__name__, "C1000", input_curve, x_axis_interval, output_curve)

    def C180(input_curve, x_axis_interval=Oasys.gRPC.defaultArg, output_curve=Oasys.gRPC.defaultArg):
        """
        SAE Class 180 Filter

        Parameters
        ----------
        input_curve : Curve
            Input Curve
        x_axis_interval : float
            Optional. If defined then T-HIS will automatically regularise the curve using this value first
        output_curve : Curve
            Optional. Curve to overwrite

        Returns
        -------
        Curve
            Curve object or None
        """
        return Oasys.THIS._connection.classMethod(__class__.__name__, "C180", input_curve, x_axis_interval, output_curve)

    def C60(input_curve, x_axis_interval=Oasys.gRPC.defaultArg, output_curve=Oasys.gRPC.defaultArg):
        """
        SAE Class 60 Filter

        Parameters
        ----------
        input_curve : Curve
            Input Curve
        x_axis_interval : float
            Optional. If defined then T-HIS will automatically regularise the curve using this value first
        output_curve : Curve
            Optional. Curve to overwrite

        Returns
        -------
        Curve
            Curve object or None
        """
        return Oasys.THIS._connection.classMethod(__class__.__name__, "C60", input_curve, x_axis_interval, output_curve)

    def C600(input_curve, x_axis_interval=Oasys.gRPC.defaultArg, output_curve=Oasys.gRPC.defaultArg):
        """
        SAE Class 600 Filter

        Parameters
        ----------
        input_curve : Curve
            Input Curve
        x_axis_interval : float
            Optional. If defined then T-HIS will automatically regularise the curve using this value first
        output_curve : Curve
            Optional. Curve to overwrite

        Returns
        -------
        Curve
            Curve object or None
        """
        return Oasys.THIS._connection.classMethod(__class__.__name__, "C600", input_curve, x_axis_interval, output_curve)

    def Cat(first_curve, second_curve, output_curve=Oasys.gRPC.defaultArg):
        """
        Concatenate 2 curves together

        Parameters
        ----------
        first_curve : Curve
            First Curve
        second_curve : Curve or real
            Second Curve
        output_curve : Curve
            Optional. Curve to overwrite

        Returns
        -------
        Curve
            Curve object or None
        """
        return Oasys.THIS._connection.classMethod(__class__.__name__, "Cat", first_curve, second_curve, output_curve)

    def Clip(input_curve, x_min, x_max, y_min, y_max, output_curve=Oasys.gRPC.defaultArg):
        """
        Clip a curve

        Parameters
        ----------
        input_curve : Curve
            Input Curve
        x_min : float
            X minimum value
        x_max : float
            X maximum value
        y_min : float
            Y minimum value
        y_max : float
            Y maximum value
        output_curve : Curve
            Optional. Curve to overwrite

        Returns
        -------
        Curve
            Curve object or None
        """
        return Oasys.THIS._connection.classMethod(__class__.__name__, "Clip", input_curve, x_min, x_max, y_min, y_max, output_curve)

    def Com(first_curve, second_curve, output_curve=Oasys.gRPC.defaultArg):
        """
        Combine Y axis values from 2 curves together

        Parameters
        ----------
        first_curve : Curve
            First Curve
        second_curve : Curve or real
            Second Curve
        output_curve : Curve
            Optional. Curve to overwrite

        Returns
        -------
        Curve
            Curve object or None
        """
        return Oasys.THIS._connection.classMethod(__class__.__name__, "Com", first_curve, second_curve, output_curve)

    def Cor(first_curve, second_curve, correlation_type):
        """
        Curve Correlation function. 
        This Correlation function provides a measure of the degree to which two curves match.
        When comparing curves by eye, the quality of correlation may be judged on the basis
        of how well matched are the patterns of peaks, the overall shapes of the curves, etc,
        and can allow for differences of timing as well as magnitude. Thus a simple function
        based on the difference of Y-values (such as T/HIS ERR function) does not measure
        correlation in the same way as the human eye. The T/HIS correlation function attempts
        to include and quantify the more subtle ways in which the correlation of two curves
        may be judged.
        The correlation can be calculated using either a strict or loose set of input parameters.
        The degree of correlation is rated between 0 and 100

        Parameters
        ----------
        first_curve : Curve
            First Curve
        second_curve : Curve
            Second Curve
        correlation_type : string
            Correlation type, strict or loose

        Returns
        -------
        float
            Correlation value
        """
        return Oasys.THIS._connection.classMethod(__class__.__name__, "Cor", first_curve, second_curve, correlation_type)

    def Cor3(first_curve, second_curve, x_axis_factor=Oasys.gRPC.defaultArg, y_axis_factor=Oasys.gRPC.defaultArg):
        """
        Curve Correlation function.
        This function first normalises the curves using two factors either specified by the user
        or defaults calculated by the program (the maximum absolute X and Y values of both
        graphs). For each point on the first normalised curve, the shortest distance to the
        second normalised curve is calculated. The root mean square value of all these
        distances is subtracted from 1 and then multiplied by 100 to get an index between 0
        and 100. The process is repeated along the second curve and the two indices are
        averaged to get a final index. The higher the index the closer the correlation between
        the two curves.
        Note that the choice of normalising factors is important. Incorrect factors may lead to a
        correlation index outside the range of 0 to 100

        Parameters
        ----------
        first_curve : Curve
            First Curve
        second_curve : Curve
            Second Curve
        x_axis_factor : float
            Optional. Normalising factor used for X axis values
        y_axis_factor : float
            Optional. Normalising factor used for Y axis values

        Returns
        -------
        float
            Correlation value
        """
        return Oasys.THIS._connection.classMethod(__class__.__name__, "Cor3", first_curve, second_curve, x_axis_factor, y_axis_factor)

    def Cos(input_curve, output_curve=Oasys.gRPC.defaultArg):
        """
        Calculate Cosine

        Parameters
        ----------
        input_curve : Curve
            Input Curve
        output_curve : Curve
            Optional. Curve to overwrite

        Returns
        -------
        Curve
            Curve object or None
        """
        return Oasys.THIS._connection.classMethod(__class__.__name__, "Cos", input_curve, output_curve)

    def Da(input_curve, output_curve=Oasys.gRPC.defaultArg):
        """
        Convert displacment spectrum to an acceleration spectrum

        Parameters
        ----------
        input_curve : Curve
            Input Curve
        output_curve : Curve
            Optional. Curve to overwrite

        Returns
        -------
        Curve
            Curve object or None
        """
        return Oasys.THIS._connection.classMethod(__class__.__name__, "Da", input_curve, output_curve)

    def Dif(input_curve, output_curve=Oasys.gRPC.defaultArg):
        """
        Differentiate a curve

        Parameters
        ----------
        input_curve : Curve
            Input Curve
        output_curve : Curve
            Optional. Curve to overwrite

        Returns
        -------
        Curve
            Curve object or None
        """
        return Oasys.THIS._connection.classMethod(__class__.__name__, "Dif", input_curve, output_curve)

    def Div(first_curve, second_curve_or_constant, output_curve=Oasys.gRPC.defaultArg):
        """
        Divide Y axis values

        Parameters
        ----------
        first_curve : Curve
            First Curve
        second_curve_or_constant : Curve or real
            Second Curve or constant
        output_curve : Curve
            Optional. Curve to overwrite

        Returns
        -------
        Curve
            Curve object or None
        """
        return Oasys.THIS._connection.classMethod(__class__.__name__, "Div", first_curve, second_curve_or_constant, output_curve)

    def Dix(first_curve, second_curve_or_constant, output_curve=Oasys.gRPC.defaultArg):
        """
        Divide X axis values

        Parameters
        ----------
        first_curve : Curve
            First Curve
        second_curve_or_constant : Curve or real
            Second Curve or constant
        output_curve : Curve
            Optional. Curve to overwrite

        Returns
        -------
        Curve
            Curve object or None
        """
        return Oasys.THIS._connection.classMethod(__class__.__name__, "Dix", first_curve, second_curve_or_constant, output_curve)

    def Ds(input_curve, broadening_factor, redefine_frequencies, output_curve=Oasys.gRPC.defaultArg):
        """
        Generate a design spectrum from a reponse spectrum

        Parameters
        ----------
        input_curve : Curve
            Input Curve
        broadening_factor : float
            Spectrum broadening factor
        redefine_frequencies : string
            T-HIS selects a new set of frequencies for the output (yes or no)
        output_curve : Curve
            Optional. Curve to overwrite

        Returns
        -------
        Curve
            Curve object or None
        """
        return Oasys.THIS._connection.classMethod(__class__.__name__, "Ds", input_curve, broadening_factor, redefine_frequencies, output_curve)

    def Dv(input_curve, output_curve=Oasys.gRPC.defaultArg):
        """
        Convert displacment spectrum to a velocity spectrum

        Parameters
        ----------
        input_curve : Curve
            Input Curve
        output_curve : Curve
            Optional. Curve to overwrite

        Returns
        -------
        Curve
            Curve object or None
        """
        return Oasys.THIS._connection.classMethod(__class__.__name__, "Dv", input_curve, output_curve)

    def Env(curves, output_curve=Oasys.gRPC.defaultArg):
        """
        Generate an Envelope that bounds the min and max values of a group of curves

        Parameters
        ----------
        curves : List of Curve objects
            List of Curve objects
        output_curve : Curve
            Optional. Curve to overwrite

        Returns
        -------
        Curve
            Curve object or None
        """
        return Oasys.THIS._connection.classMethod(__class__.__name__, "Env", curves, output_curve)

    def Err(first_curve, second_curve, output_curve=Oasys.gRPC.defaultArg):
        """
        Calculate the degree of correlation between 2 curves

        Parameters
        ----------
        first_curve : Curve
            First Curve
        second_curve : Curve or real
            Second Curve
        output_curve : Curve
            Optional. Curve to overwrite

        Returns
        -------
        Curve
            Curve object or None
        """
        return Oasys.THIS._connection.classMethod(__class__.__name__, "Err", first_curve, second_curve, output_curve)

    def Exc(input_curve, output_option, output_curve=Oasys.gRPC.defaultArg):
        """
        Calculate and displays an EXCeedence plot. This is a plot of force (Y axis) versus 
        cumulative time (X axis) for which the force level has been exceeded. By default the Automatic option 
        will create an exceedence plot using either the +ve OR the -ve values depending on which the input 
        curve contains most of. 
        The Positive option will calculate the exceedence plot using only the points with +ve y values.
        The Negative option will calculate the exceedence plot using only the points with -ve y values

        Parameters
        ----------
        input_curve : Curve
            Input Curve
        output_option : string
            Select between automatic, positive or negative
        output_curve : Curve
            Optional. Curve to overwrite

        Returns
        -------
        Curve
            Curve object or None
        """
        return Oasys.THIS._connection.classMethod(__class__.__name__, "Exc", input_curve, output_option, output_curve)

    def Exp(input_curve, output_curve=Oasys.gRPC.defaultArg):
        """
        Calculate E to the power of Y axis values

        Parameters
        ----------
        input_curve : Curve
            Input Curve
        output_curve : Curve
            Optional. Curve to overwrite

        Returns
        -------
        Curve
            Curve object or None
        """
        return Oasys.THIS._connection.classMethod(__class__.__name__, "Exp", input_curve, output_curve)

    def Fft(input_curve, output_option, x_axis_interval=Oasys.gRPC.defaultArg, scaling_option=Oasys.gRPC.defaultArg):
        """
        Fast Fourier Transform

        Parameters
        ----------
        input_curve : Curve
            Input Curve
        output_option : string
            Generate magnitude, magnitude+phase or real+imaginary, (one of magnitude,phase,real)
        x_axis_interval : float
            Optional. If defined then T-HIS will automatically regularise the curve using this value first
        scaling_option : string
            Optional. Scaling option, (either one or two)

        Returns
        -------
        Curve
            Curve object/list or None
        """
        return Oasys.THIS._connection.classMethod(__class__.__name__, "Fft", input_curve, output_option, x_axis_interval, scaling_option)

    def Fir(input_curve, x_axis_interval=Oasys.gRPC.defaultArg, output_curve=Oasys.gRPC.defaultArg):
        """
        FIR Filter

        Parameters
        ----------
        input_curve : Curve
            Input Curve
        x_axis_interval : float
            Optional. If defined then T-HIS will automatically regularise the curve using this value first
        output_curve : Curve
            Optional. Curve to overwrite

        Returns
        -------
        Curve
            Curve object or None
        """
        return Oasys.THIS._connection.classMethod(__class__.__name__, "Fir", input_curve, x_axis_interval, output_curve)

    def Hic(input_curve, window, acceleration_factor):
        """
        HIC Calculation. After calculating the HIC value for a curve the value can also be obtained
        from the curve using the Curve.hic property. In addition to the HIC 
        value the start and end time for the time window can also be obtained using the Curve.hic_tmin
        and Curve.hic_tmax properties

        Parameters
        ----------
        input_curve : Curve
            Input Curve
        window : float
            Maximum time window
        acceleration_factor : float
            Factor required to divide input acceleration curve by to convert to (G)

        Returns
        -------
        float
            HIC value
        """
        return Oasys.THIS._connection.classMethod(__class__.__name__, "Hic", input_curve, window, acceleration_factor)

    def Hicd(input_curve, window, acceleration_factor):
        """
        Modified HIC(d) Calculation for free motion headform. After calculating the HIC value for a curve 
        the value can also be obtained from the curve using the Curve.hicd property. 
        In addition to the HIC(d) value the start and end time for the time window can also be obtained using the 
        Curve.hicd_tmin and 
        Curve.hicd_tmax properties

        Parameters
        ----------
        input_curve : Curve
            Input Curve
        window : float
            Maximum time window
        acceleration_factor : float
            Factor required to divide input acceleration curve by to convert to (G)

        Returns
        -------
        float
            HIC(d) value
        """
        return Oasys.THIS._connection.classMethod(__class__.__name__, "Hicd", input_curve, window, acceleration_factor)

    def Ifft(first_curve, second_curve, input_type):
        """
        Inverse Fast Fourier Transform

        Parameters
        ----------
        first_curve : Curve
            First Curve
        second_curve : Curve
            Second Curve
        input_type : string
            Specifies if inputs are magnitude+phase or real+imaginary, (magnitude or real)

        Returns
        -------
        Curve
            Curve object or None
        """
        return Oasys.THIS._connection.classMethod(__class__.__name__, "Ifft", first_curve, second_curve, input_type)

    def Int(input_curve, output_curve=Oasys.gRPC.defaultArg):
        """
        Integrate a curve

        Parameters
        ----------
        input_curve : Curve
            Input Curve
        output_curve : Curve
            Optional. Curve to overwrite

        Returns
        -------
        Curve
            Curve object or None
        """
        return Oasys.THIS._connection.classMethod(__class__.__name__, "Int", input_curve, output_curve)

    def Log(input_curve, output_curve=Oasys.gRPC.defaultArg):
        """
        Calculate Natural Log of Y axis values

        Parameters
        ----------
        input_curve : Curve
            Input Curve
        output_curve : Curve
            Optional. Curve to overwrite

        Returns
        -------
        Curve
            Curve object or None
        """
        return Oasys.THIS._connection.classMethod(__class__.__name__, "Log", input_curve, output_curve)

    def Log10(input_curve, output_curve=Oasys.gRPC.defaultArg):
        """
        Calculate Log (base 10) of Y axis values

        Parameters
        ----------
        input_curve : Curve
            Input Curve
        output_curve : Curve
            Optional. Curve to overwrite

        Returns
        -------
        Curve
            Curve object or None
        """
        return Oasys.THIS._connection.classMethod(__class__.__name__, "Log10", input_curve, output_curve)

    def Log10x(input_curve, output_curve=Oasys.gRPC.defaultArg):
        """
        Calculate Log (base 10) of X axis values

        Parameters
        ----------
        input_curve : Curve
            Input Curve
        output_curve : Curve
            Optional. Curve to overwrite

        Returns
        -------
        Curve
            Curve object or None
        """
        return Oasys.THIS._connection.classMethod(__class__.__name__, "Log10x", input_curve, output_curve)

    def Logx(input_curve, output_curve=Oasys.gRPC.defaultArg):
        """
        Calculate Natural Log of X axis values

        Parameters
        ----------
        input_curve : Curve
            Input Curve
        output_curve : Curve
            Optional. Curve to overwrite

        Returns
        -------
        Curve
            Curve object or None
        """
        return Oasys.THIS._connection.classMethod(__class__.__name__, "Logx", input_curve, output_curve)

    def Lsq(input_curve, output_curve=Oasys.gRPC.defaultArg):
        """
        Calculate Least Squares Fit for a curve

        Parameters
        ----------
        input_curve : Curve
            Input Curve
        output_curve : Curve
            Optional. Curve to overwrite

        Returns
        -------
        Curve
            Curve object or None
        """
        return Oasys.THIS._connection.classMethod(__class__.__name__, "Lsq", input_curve, output_curve)

    def Map(first_curve, second_curve, output_curve=Oasys.gRPC.defaultArg):
        """
        Map Y axis values from one curve onto another curve

        Parameters
        ----------
        first_curve : Curve
            First Curve
        second_curve : Curve or real
            Second Curve
        output_curve : Curve
            Optional. Curve to overwrite

        Returns
        -------
        Curve
            Curve object or None
        """
        return Oasys.THIS._connection.classMethod(__class__.__name__, "Map", first_curve, second_curve, output_curve)

    def Max(curves, output_curve=Oasys.gRPC.defaultArg):
        """
        Maximum of a group of curves

        Parameters
        ----------
        curves : List of Curve objects
            List of Curve objects
        output_curve : Curve
            Optional. Curve to overwrite

        Returns
        -------
        Curve
            Curve object or None
        """
        return Oasys.THIS._connection.classMethod(__class__.__name__, "Max", curves, output_curve)

    def Min(curves, output_curve=Oasys.gRPC.defaultArg):
        """
        Minimum of a group of curves

        Parameters
        ----------
        curves : List of Curve objects
            List of Curve objects
        output_curve : Curve
            Optional. Curve to overwrite

        Returns
        -------
        Curve
            Curve object or None
        """
        return Oasys.THIS._connection.classMethod(__class__.__name__, "Min", curves, output_curve)

    def Mon(input_curve, output_curve=Oasys.gRPC.defaultArg):
        """
        Sort a curve into monotonically increasing X axis values

        Parameters
        ----------
        input_curve : Curve
            Input Curve
        output_curve : Curve
            Optional. Curve to overwrite

        Returns
        -------
        Curve
            Curve object or None
        """
        return Oasys.THIS._connection.classMethod(__class__.__name__, "Mon", input_curve, output_curve)

    def Mul(first_curve, second_curve_or_constant, output_curve=Oasys.gRPC.defaultArg):
        """
        Multiply Y axis values

        Parameters
        ----------
        first_curve : Curve
            First Curve
        second_curve_or_constant : Curve or real
            Second Curve or constant
        output_curve : Curve
            Optional. Curve to overwrite

        Returns
        -------
        Curve
            Curve object or None
        """
        return Oasys.THIS._connection.classMethod(__class__.__name__, "Mul", first_curve, second_curve_or_constant, output_curve)

    def Mux(first_curve, second_curve_or_constant, output_curve=Oasys.gRPC.defaultArg):
        """
        Multiply X axis values

        Parameters
        ----------
        first_curve : Curve
            First Curve
        second_curve_or_constant : Curve or real
            Second Curve or constant
        output_curve : Curve
            Optional. Curve to overwrite

        Returns
        -------
        Curve
            Curve object or None
        """
        return Oasys.THIS._connection.classMethod(__class__.__name__, "Mux", first_curve, second_curve_or_constant, output_curve)

    def Ncp(first_curve, second_curve):
        """
        Calculate a platic rotation curve for a beam from a moment/time and rotation/time

        Parameters
        ----------
        first_curve : Curve
            Moment / Time Curve
        second_curve : Curve
            Rotation /Time Curve

        Returns
        -------
        Curve
            Curve object or None
        """
        return Oasys.THIS._connection.classMethod(__class__.__name__, "Ncp", first_curve, second_curve)

    def Nij(shear_force, axial_force, moment, fzc_t, fzc_c, myc_f, myc_e, e):
        """
        Biomechanical neck injury predictor. Used as a measure of injury due to the 
        load transferred through the occipital condyles.
        This function returns a list containing 4 curve objects.
        Curve 1 - "Nte" is the tension-extension condition
        Curve 2 - "Ntf" is the tension-flexion condition
        Curve 3 - "Nce" is the compression-extension condition
        Curve 4 - "Ncf" is the compression-flexion condition

        Parameters
        ----------
        shear_force : Curve
            Shear Force Curve
        axial_force : Curve
            Axial Force Curve
        moment : Curve
            Moment Curve
        fzc_t : float
            Critical Axial Force (Tension)
        fzc_c : float
            Critical Axial Force (Compression)
        myc_f : float
            Critical bending moment (Flexion)
        myc_e : float
            Critical bending moment (Extension)
        e : float
            Distance

        Returns
        -------
        list
            List of Curve objects.
            1st curve : Nte curve
            2nd curve : Ntf curve
            3rd curve : Nce curve
            4th curve : Ncf curve
        """
        return Oasys.THIS._connection.classMethod(__class__.__name__, "Nij", shear_force, axial_force, moment, fzc_t, fzc_c, myc_f, myc_e, e)

    def Nor(input_curve, output_curve=Oasys.gRPC.defaultArg):
        """
        Normalise Y axis values between [-1,1]

        Parameters
        ----------
        input_curve : Curve
            Input Curve
        output_curve : Curve
            Optional. Curve to overwrite

        Returns
        -------
        Curve
            Curve object or None
        """
        return Oasys.THIS._connection.classMethod(__class__.__name__, "Nor", input_curve, output_curve)

    def Nor2(input_curve, y_min_value, y_max_value, lock_to_axis_y_min, lock_to_axis_y_max, output_curve=Oasys.gRPC.defaultArg):
        """
        Normalise Y axis values with manual settings. The operation takes 
        the absolute value of the user-specified Y Min and Y Max. It then finds the 
        maximum of these two numbers and divides all Y data by this number. There 
        are two locks which probe or "lock on to" the Y Max and Y Min axis values which offers 
        quick axis-normalizing

        Parameters
        ----------
        input_curve : Curve
            First Curve
        y_min_value : float
            The Minimum Y value
        y_max_value : float
            The Maximum Y value
        lock_to_axis_y_min : integer
            Set the Lock button for the Y Minimum textbox
        lock_to_axis_y_max : integer
            Set the Lock button for the Y Maximum textbox
        output_curve : Curve
            Optional. Curve to overwrite

        Returns
        -------
        Curve
            Curve object or None
        """
        return Oasys.THIS._connection.classMethod(__class__.__name__, "Nor2", input_curve, y_min_value, y_max_value, lock_to_axis_y_min, lock_to_axis_y_max, output_curve)

    def Nox(input_curve, output_curve=Oasys.gRPC.defaultArg):
        """
        Normalise X axis values between [-1,1]

        Parameters
        ----------
        input_curve : Curve
            Input Curve
        output_curve : Curve
            Optional. Curve to overwrite

        Returns
        -------
        Curve
            Curve object or None
        """
        return Oasys.THIS._connection.classMethod(__class__.__name__, "Nox", input_curve, output_curve)

    def Nox2(input_curve, x_min_value, x_max_value, lock_to_axis_x_min, lock_to_axis_x_max, output_curve=Oasys.gRPC.defaultArg):
        """
        Normalise X axis values with manual settings. The operation takes 
        the absolute value of the user-specified X Min and X Max. It then finds the 
        maximum of these two numbers and divides all X data by this number. There 
        are two locks which probe or "lock on to" the X Max and X Min axis values which offers 
        quick axis-normalizing

        Parameters
        ----------
        input_curve : Curve
            First Curve
        x_min_value : float
            The Minimum X value
        x_max_value : float
            The Maximum X value
        lock_to_axis_x_min : integer
            Set the Lock button for the X Minimum textbox
        lock_to_axis_x_max : integer
            Set the Lock button for the X Maximum textbox
        output_curve : Curve
            Optional. Curve to overwrite

        Returns
        -------
        Curve
            Curve object or None
        """
        return Oasys.THIS._connection.classMethod(__class__.__name__, "Nox2", input_curve, x_min_value, x_max_value, lock_to_axis_x_min, lock_to_axis_x_max, output_curve)

    def Octave(input_curve, band_type_to_convert_to, output_type, input_type, output_curve=Oasys.gRPC.defaultArg):
        """
        Coverts a narrow band curve to either Octave or 1/Third Octave bands

        Parameters
        ----------
        input_curve : Curve
            Input Curve
        band_type_to_convert_to : String
            Band type to convert to. Either "Octave" or "Third" Octave
        output_type : String
            Generate curve containing either "RMS" or "mean" values
        input_type : String
            Input curve contains either "Linear" or "dB" values
        output_curve : Curve
            Optional. Curve to overwrite

        Returns
        -------
        Curve
            Curve object or None
        """
        return Oasys.THIS._connection.classMethod(__class__.__name__, "Octave", input_curve, band_type_to_convert_to, output_type, input_type, output_curve)

    def Order(input_curve, output_curve=Oasys.gRPC.defaultArg):
        """
        Reverse the order of points in a curve

        Parameters
        ----------
        input_curve : Curve
            Input Curve
        output_curve : Curve
            Optional. Curve to overwrite

        Returns
        -------
        Curve
            Curve object or None
        """
        return Oasys.THIS._connection.classMethod(__class__.__name__, "Order", input_curve, output_curve)

    def Pbut(input_curve, frequency, order, x_axis_interval=Oasys.gRPC.defaultArg, output_curve=Oasys.gRPC.defaultArg):
        """
        Pure Butterworth Filter

        Parameters
        ----------
        input_curve : Curve
            Input Curve
        frequency : float
            Cut-off Frequency (Hz)
        order : integer
            Filter order
        x_axis_interval : float
            Optional. If defined then T-HIS will automatically regularise the curve using this value first
        output_curve : Curve
            Optional. Curve to overwrite

        Returns
        -------
        Curve
            Curve object or None
        """
        return Oasys.THIS._connection.classMethod(__class__.__name__, "Pbut", input_curve, frequency, order, x_axis_interval, output_curve)

    def Power(input_curve, power, output_curve=Oasys.gRPC.defaultArg):
        """
        Raise to the power

        Parameters
        ----------
        input_curve : Curve
            Input Curve
        power : float
            Power to raise Y axis values by
        output_curve : Curve
            Optional. Curve to overwrite

        Returns
        -------
        Curve
            Curve object or None
        """
        return Oasys.THIS._connection.classMethod(__class__.__name__, "Power", input_curve, power, output_curve)

    def Rave(input_curve, output_curve=Oasys.gRPC.defaultArg):
        """
        Calculate rolling average of a curve

        Parameters
        ----------
        input_curve : Curve
            Input Curve
        output_curve : Curve
            Optional. Curve to overwrite

        Returns
        -------
        Curve
            Curve object or None
        """
        return Oasys.THIS._connection.classMethod(__class__.__name__, "Rave", input_curve, output_curve)

    def Rec(input_curve, output_curve=Oasys.gRPC.defaultArg):
        """
        Calculate reciprocal

        Parameters
        ----------
        input_curve : Curve
            Input Curve
        output_curve : Curve
            Optional. Curve to overwrite

        Returns
        -------
        Curve
            Curve object or None
        """
        return Oasys.THIS._connection.classMethod(__class__.__name__, "Rec", input_curve, output_curve)

    def Reg(input_curve, x_axis_interval, output_curve=Oasys.gRPC.defaultArg):
        """
        Regularise X axis intervals for a curve

        Parameters
        ----------
        input_curve : Curve
            Input Curve
        x_axis_interval : float
            New X axis interval
        output_curve : Curve
            Optional. Curve to overwrite

        Returns
        -------
        Curve
            Curve object or None
        """
        return Oasys.THIS._connection.classMethod(__class__.__name__, "Reg", input_curve, x_axis_interval, output_curve)

    def Res(curves, output_curve=Oasys.gRPC.defaultArg):
        """
        Resultant of a group of curves

        Parameters
        ----------
        curves : List of Curve objects
            List of Curve objects
        output_curve : Curve
            Optional. Curve to overwrite

        Returns
        -------
        Curve
            Curve object or None
        """
        return Oasys.THIS._connection.classMethod(__class__.__name__, "Res", curves, output_curve)

    def Rev(input_curve, output_curve=Oasys.gRPC.defaultArg):
        """
        Reverse X and Y axis values

        Parameters
        ----------
        input_curve : Curve
            Input Curve
        output_curve : Curve
            Optional. Curve to overwrite

        Returns
        -------
        Curve
            Curve object or None
        """
        return Oasys.THIS._connection.classMethod(__class__.__name__, "Rev", input_curve, output_curve)

    def Rs(input_curve, damping_factor, sampling_points, x_axis_interval=Oasys.gRPC.defaultArg, output_curve=Oasys.gRPC.defaultArg):
        """
        Generate a reponse spectrum from input accelerations

        Parameters
        ----------
        input_curve : Curve
            Input Curve
        damping_factor : float
            Dammping factor
        sampling_points : int
            Number of points to sample over (30 or 70)
        x_axis_interval : float
            Optional. If defined then T-HIS will automatically regularise the curve using this value first
        output_curve : Curve
            Optional. Curve to overwrite

        Returns
        -------
        list
            List of Curve objects
            1st curve : Relative displacement
            2nd curve : Relative velocity
            3th curve : Pseudo relative velocity
            4th curve : Absolute acceleration
            5th curve : Pseudo absolute acceleration
        """
        return Oasys.THIS._connection.classMethod(__class__.__name__, "Rs", input_curve, damping_factor, sampling_points, x_axis_interval, output_curve)

    def Sin(input_curve, output_curve=Oasys.gRPC.defaultArg):
        """
        Calculate Sine

        Parameters
        ----------
        input_curve : Curve
            Input Curve
        output_curve : Curve
            Optional. Curve to overwrite

        Returns
        -------
        Curve
            Curve object or None
        """
        return Oasys.THIS._connection.classMethod(__class__.__name__, "Sin", input_curve, output_curve)

    def Smooth(input_curve, smoothing_factor, output_curve=Oasys.gRPC.defaultArg):
        """
        Apply a smoothing factor to a curve

        Parameters
        ----------
        input_curve : Curve
            Input Curve
        smoothing_factor : integer
            Number of points to average over
        output_curve : Curve
            Optional. Curve to overwrite

        Returns
        -------
        Curve
            Curve object or None
        """
        return Oasys.THIS._connection.classMethod(__class__.__name__, "Smooth", input_curve, smoothing_factor, output_curve)

    def Sqr(input_curve, output_curve=Oasys.gRPC.defaultArg):
        """
        Square root of a curve

        Parameters
        ----------
        input_curve : Curve
            Input Curve
        output_curve : Curve
            Optional. Curve to overwrite

        Returns
        -------
        Curve
            Curve object or None
        """
        return Oasys.THIS._connection.classMethod(__class__.__name__, "Sqr", input_curve, output_curve)

    def Stress(input_curve, convert_to, output_curve=Oasys.gRPC.defaultArg):
        """
        Convert between true and engineering stress

        Parameters
        ----------
        input_curve : Curve
            Input Curve
        convert_to : string
            Type to convert to (True or Engineering)
        output_curve : Curve
            Optional. Curve to overwrite

        Returns
        -------
        Curve
            Curve object or None
        """
        return Oasys.THIS._connection.classMethod(__class__.__name__, "Stress", input_curve, convert_to, output_curve)

    def Sub(first_curve, second_curve_or_constant, output_curve=Oasys.gRPC.defaultArg):
        """
        Subtract Y axis values

        Parameters
        ----------
        first_curve : Curve
            First Curve
        second_curve_or_constant : Curve or real
            Second Curve or constant
        output_curve : Curve
            Optional. Curve to overwrite

        Returns
        -------
        Curve
            Curve object or None
        """
        return Oasys.THIS._connection.classMethod(__class__.__name__, "Sub", first_curve, second_curve_or_constant, output_curve)

    def Sum(curves, output_curve=Oasys.gRPC.defaultArg):
        """
        Sum of a group of curves

        Parameters
        ----------
        curves : List of Curve objects
            List of Curve objects
        output_curve : Curve
            Optional. Curve to overwrite

        Returns
        -------
        Curve
            Curve object or None
        """
        return Oasys.THIS._connection.classMethod(__class__.__name__, "Sum", curves, output_curve)

    def Sux(first_curve, second_curve_or_constant, output_curve=Oasys.gRPC.defaultArg):
        """
        Subtract X axis values

        Parameters
        ----------
        first_curve : Curve
            First Curve
        second_curve_or_constant : Curve or real
            Second Curve or constant
        output_curve : Curve
            Optional. Curve to overwrite

        Returns
        -------
        Curve
            Curve object or None
        """
        return Oasys.THIS._connection.classMethod(__class__.__name__, "Sux", first_curve, second_curve_or_constant, output_curve)

    def Tan(input_curve, output_curve=Oasys.gRPC.defaultArg):
        """
        Calculate Tangent

        Parameters
        ----------
        input_curve : Curve
            Input Curve
        output_curve : Curve
            Optional. Curve to overwrite

        Returns
        -------
        Curve
            Curve object or None
        """
        return Oasys.THIS._connection.classMethod(__class__.__name__, "Tan", input_curve, output_curve)

    def Thiv(x_acceleration, y_acceleration, yaw_rate, dx, dy, x0):
        """
        Theoretical Head Impact Velocity and the Post Impact Head Deceleration.
        These values are used to assess the performance of road side crash barriers.
        This function returns a list containing 2 curve objects. The 1st curve is the THIV
        curve and the 2nd is the PHD curve. The peak values of these curves are the corresponding
        THIV and PHD values and can be obtained using the Curve.ymax
        property

        Parameters
        ----------
        x_acceleration : Curve
            X Acceleration Curve
        y_acceleration : Curve
            Y Acceleration Curve
        yaw_rate : Curve
            Yaw Rate Curve
        dx : float
            Horizontal distance between occupants head and vehicle
        dy : float
            Lateral distance between occupants head and vehicle
        x0 : float
            Horizontal distance between occupants head and vehicle CofG

        Returns
        -------
        list
            List of Curve objects.
            1st curve : THIV curve
            2nd curve : PHD curve
        """
        return Oasys.THIS._connection.classMethod(__class__.__name__, "Thiv", x_acceleration, y_acceleration, yaw_rate, dx, dy, x0)

    def Tms(input_curve, period):
        """
        3ms Clip Calculation. After calculating the 3ms clip value for a curve the value can also be obtained
        from the curve using the Curve.tms property. In addition to the 3ms clip value
        the start and end time for the time window can also be obtained using the Curve.tms_tmin
        and Curve.tms_tmax properties

        Parameters
        ----------
        input_curve : Curve
            Input Curve
        period : float
            Clip period

        Returns
        -------
        float
            3ms clip value
        """
        return Oasys.THIS._connection.classMethod(__class__.__name__, "Tms", input_curve, period)

    def Translate(input_curve, x_value, y_value, output_curve=Oasys.gRPC.defaultArg):
        """
        Translate a curve

        Parameters
        ----------
        input_curve : Curve
            Input Curve
        x_value : float
            X translation value
        y_value : float
            Y translation value
        output_curve : Curve
            Optional. Curve to overwrite

        Returns
        -------
        Curve
            Curve object or None
        """
        return Oasys.THIS._connection.classMethod(__class__.__name__, "Translate", input_curve, x_value, y_value, output_curve)

    def Tti(upper_rib_acceleration, lower_rib_acceleration, t12_acceleration):
        """
        Thorax Trauma Index

        Parameters
        ----------
        upper_rib_acceleration : Curve
            Upper Rib Acceleration Curve
        lower_rib_acceleration : Curve
            Lower Rib Acceleration Curve
        t12_acceleration : Curve
            T12 Acceleration Curve

        Returns
        -------
        float
            TTI value
        """
        return Oasys.THIS._connection.classMethod(__class__.__name__, "Tti", upper_rib_acceleration, lower_rib_acceleration, t12_acceleration)

    def Va(input_curve, output_curve=Oasys.gRPC.defaultArg):
        """
        Convert velocity spectrum to an acceleration spectrum

        Parameters
        ----------
        input_curve : Curve
            Input Curve
        output_curve : Curve
            Optional. Curve to overwrite

        Returns
        -------
        Curve
            Curve object or None
        """
        return Oasys.THIS._connection.classMethod(__class__.__name__, "Va", input_curve, output_curve)

    def Vc(input_curve, a, b, calculation_method, output_curve=Oasys.gRPC.defaultArg):
        """
        Viscous Criteria calculate. The VC calculation can be done using 2 different calculation 
        methods ECER95 and IIHS

        Parameters
        ----------
        input_curve : Curve
            Input Curve
        a : float
            Constant A
        b : float
            Constant B
        calculation_method : string
            Either ECER95 or IIHS
        output_curve : Curve
            Optional. Curve to overwrite

        Returns
        -------
        Curve
            Curve object or None
        """
        return Oasys.THIS._connection.classMethod(__class__.__name__, "Vc", input_curve, a, b, calculation_method, output_curve)

    def Vd(input_curve, output_curve=Oasys.gRPC.defaultArg):
        """
        Convert velocity spectrum to a displacment spectrum

        Parameters
        ----------
        input_curve : Curve
            Input Curve
        output_curve : Curve
            Optional. Curve to overwrite

        Returns
        -------
        Curve
            Curve object or None
        """
        return Oasys.THIS._connection.classMethod(__class__.__name__, "Vd", input_curve, output_curve)

    def Vec(first_curve, second_curve, third_curve, output_curve=Oasys.gRPC.defaultArg):
        """
        Vector magnitude of 3 curves

        Parameters
        ----------
        first_curve : Curve
            First Curve
        second_curve : Curve or real
            Second Curve
        third_curve : Curve or real
            Second Curve
        output_curve : Curve
            Optional. Curve to overwrite

        Returns
        -------
        Curve
            Curve object or None
        """
        return Oasys.THIS._connection.classMethod(__class__.__name__, "Vec", first_curve, second_curve, third_curve, output_curve)

    def Vec2d(first_curve, second_curve, output_curve=Oasys.gRPC.defaultArg):
        """
        Vector magnitude of 2 curves

        Parameters
        ----------
        first_curve : Curve
            First Curve
        second_curve : Curve or real
            Second Curve
        output_curve : Curve
            Optional. Curve to overwrite

        Returns
        -------
        Curve
            Curve object or None
        """
        return Oasys.THIS._connection.classMethod(__class__.__name__, "Vec2d", first_curve, second_curve, output_curve)

    def Wif(first_curve, second_curve):
        """
        Weigthed Integrated Factor (WIFAC) Correlation function

        Parameters
        ----------
        first_curve : Curve
            First Curve
        second_curve : Curve
            Second Curve

        Returns
        -------
        float
            Correlation value
        """
        return Oasys.THIS._connection.classMethod(__class__.__name__, "Wif", first_curve, second_curve)

    def Window(input_curve, window_type, percentage_lead_in=Oasys.gRPC.defaultArg, output_curve=Oasys.gRPC.defaultArg):
        """
        Apply a smoothing window to a curve

        Parameters
        ----------
        input_curve : Curve
            Input Curve
        window_type : string
            Window type to apply (Hanning, cosine or exponetial)
        percentage_lead_in : float
            Optional. percentage lead in for cosine window
        output_curve : Curve
            Optional. Curve to overwrite

        Returns
        -------
        Curve
            Curve object or None
        """
        return Oasys.THIS._connection.classMethod(__class__.__name__, "Window", input_curve, window_type, percentage_lead_in, output_curve)

    def Zero(input_curve, output_curve=Oasys.gRPC.defaultArg):
        """
        Translate curve to 0,0

        Parameters
        ----------
        input_curve : Curve
            Input Curve
        output_curve : Curve
            Optional. Curve to overwrite

        Returns
        -------
        Curve
            Curve object or None
        """
        return Oasys.THIS._connection.classMethod(__class__.__name__, "Zero", input_curve, output_curve)

    def ZeroX(input_curve, output_curve=Oasys.gRPC.defaultArg):
        """
        Translate curve to X=0.0

        Parameters
        ----------
        input_curve : Curve
            Input Curve
        output_curve : Curve
            Optional. Curve to overwrite

        Returns
        -------
        Curve
            Curve object or None
        """
        return Oasys.THIS._connection.classMethod(__class__.__name__, "ZeroX", input_curve, output_curve)

    def ZeroY(input_curve, output_curve=Oasys.gRPC.defaultArg):
        """
        Translate curve to Y=0.0

        Parameters
        ----------
        input_curve : Curve
            Input Curve
        output_curve : Curve
            Optional. Curve to overwrite

        Returns
        -------
        Curve
            Curve object or None
        """
        return Oasys.THIS._connection.classMethod(__class__.__name__, "ZeroY", input_curve, output_curve)

    def dB(input_curve, reference_value, output_curve=Oasys.gRPC.defaultArg):
        """
        Converts a curve to dB (y = 20.0\*log(y/yref))

        Parameters
        ----------
        input_curve : Curve
            Input Curve
        reference_value : float
            Reference value
        output_curve : Curve
            Optional. Curve to overwrite

        Returns
        -------
        Curve
            Curve object or None
        """
        return Oasys.THIS._connection.classMethod(__class__.__name__, "dB", input_curve, reference_value, output_curve)

    def dBA(input_curve, weighting_type, output_curve=Oasys.gRPC.defaultArg):
        """
        Applies A-weighting to a curve (convert from dB to dBA)

        Parameters
        ----------
        input_curve : Curve
            Input Curve
        weighting_type : String
            Apply either Narrow band (narrow) or Octave band (octave) A weighting
        output_curve : Curve
            Optional. Curve to overwrite

        Returns
        -------
        Curve
            Curve object or None
        """
        return Oasys.THIS._connection.classMethod(__class__.__name__, "dBA", input_curve, weighting_type, output_curve)




import Oasys.gRPC


# Metaclass for static properties and constants
class OptionsType(type):
    _props = {'auto_confirm', 'max_widgets', 'max_window_lines', 'ssh_buffer_size'}

    def __getattr__(cls, name):
        if name in OptionsType._props:
            return Oasys.THIS._connection.classGetter(cls.__name__, name)

        raise AttributeError


class Options(Oasys.gRPC.OasysItem, metaclass=OptionsType):


    def __del__(self):
        if not Oasys.THIS._connection:
            return

        Oasys.THIS._connection.destructor(self.__class__.__name__, self._handle)


    def __getattr__(self, name):
        raise AttributeError


    def __setattr__(self, name, value):
# Set the property locally
        self.__dict__[name] = value



import Oasys.gRPC


# Metaclass for static properties and constants
class DatumType(type):
    _consts = {'CONSTANT_X', 'CONSTANT_Y', 'CONSTANT_Y2', 'FILL_ABOVE_BELOW', 'FILL_RIGHT_LEFT', 'LABEL_10_POINT', 'LABEL_12_POINT', 'LABEL_14_POINT', 'LABEL_18_POINT', 'LABEL_24_POINT', 'LABEL_8_POINT', 'LABEL_ABOVE_CENTRE', 'LABEL_ABOVE_LEFT', 'LABEL_ABOVE_RIGHT', 'LABEL_AUTOMATIC', 'LABEL_BELOW_CENTRE', 'LABEL_BELOW_LEFT', 'LABEL_BELOW_RIGHT', 'LABEL_BOTTOM_LEFT', 'LABEL_BOTTOM_RIGHT', 'LABEL_COURIER_BOLD', 'LABEL_COURIER_MEDIUM', 'LABEL_DEFAULT', 'LABEL_HELVETICA_BOLD', 'LABEL_HELVETICA_MEDIUM', 'LABEL_HORIZONTAL', 'LABEL_MIDDLE_LEFT', 'LABEL_MIDDLE_RIGHT', 'LABEL_NONE', 'LABEL_TIMES_BOLD', 'LABEL_TIMES_MEDIUM', 'LABEL_TOP_LEFT', 'LABEL_TOP_RIGHT', 'LABEL_VERTICAL', 'POINTS'}

    def __getattr__(cls, name):
        if name in DatumType._consts:
            return Oasys.THIS._connection.classGetter(cls.__name__, name)

        raise AttributeError


class Datum(Oasys.gRPC.OasysItem, metaclass=DatumType):
    _props = {'acronym', 'fill_colour_above', 'fill_colour_below', 'fill_colour_between', 'fill_colour_left', 'fill_colour_right', 'fill_type', 'label', 'label2', 'label_colour', 'label_font', 'label_orientation', 'label_position', 'label_size', 'line_colour', 'line_style', 'line_width'}


    def __del__(self):
        if not Oasys.THIS._connection:
            return

        Oasys.THIS._connection.destructor(self.__class__.__name__, self._handle)


    def __getattr__(self, name):
# If one of the properties we define then get it
        if name in Datum._props:
            return Oasys.THIS._connection.instanceGetter(self.__class__.__name__, self._handle, name)

        raise AttributeError


    def __setattr__(self, name, value):
# If one of the properties we define then set it
        if name in Datum._props:
            Oasys.THIS._connection.instanceSetter(self.__class__.__name__, self._handle, name, value)
            return

# Set the property locally
        self.__dict__[name] = value


# Constructor
    def __init__(self, acronym, type, value, second_value=Oasys.gRPC.defaultArg):
        handle = Oasys.THIS._connection.constructor(self.__class__.__name__, acronym, type, value, second_value)
        Oasys.gRPC.OasysItem.__init__(self, self.__class__.__name__, handle)
        """
        Create a new Datum object. 
        The datum will be added to all the currently active graphs

        Parameters
        ----------
        acronym : string
            Datum acronym
        type : integer
            Specify type of datum line. Can be Datum.CONSTANT_X, 
            Datum.CONSTANT_Y,
            Datum.CONSTANT_Y2,
            Datum.POINTS
        value : real or list of reals
            Value for Datum.CONSTANT_X,
            Datum.CONSTANT_Y or 
            Datum.CONSTANT_Y2 type Datum. 
            If it is a Datum.POINTS type Datum
            then this should be a list of X, Y pairs or a curve ID to copy points from
        second_value : real
            Optional. Second constant value for use with constant X,Y or Y2 datums and can optionally be provided

        Returns
        -------
        Datum
            Datum object
        """


# Static methods
    def Delete(datum):
        """
        Deletes a datum

        Parameters
        ----------
        datum : string
            Acronym of datum to delete

        Returns
        -------
        None
            No return value
        """
        return Oasys.THIS._connection.classMethod(__class__.__name__, "Delete", datum)

    def Exists(datum):
        """
        Checks if a datum exists

        Parameters
        ----------
        datum : string
            Acronym of datum to check

        Returns
        -------
        bool
            True if the datum exists, otherwise False
        """
        return Oasys.THIS._connection.classMethod(__class__.__name__, "Exists", datum)

    def First():
        """
        Returns the first datum

        Returns
        -------
        Datum
            Datum object (or None if there are no datum in the model)
        """
        return Oasys.THIS._connection.classMethod(__class__.__name__, "First")

    def GetFromAcronym(datum):
        """
        Returns the datum object for a datum acronym

        Parameters
        ----------
        datum : string
            Acronym of datum to return object for

        Returns
        -------
        Datum
            Datum object (or None if the datum does not exist)
        """
        return Oasys.THIS._connection.classMethod(__class__.__name__, "GetFromAcronym", datum)



# Instance methods
    def AddToGraph(self, *graph):
        """
        Adds a datum to a graph

        Parameters
        ----------
        *graph : int
            Graph to add the datum to. If undefined then the datum is added to all graphs
            This argument can be repeated if required.
            Alternatively a single array argument containing the multiple values can be given

        Returns
        -------
        None
            No return value
        """
        return Oasys.THIS._connection.instanceMethod(self.__class__.__name__, self._handle, "AddToGraph", graph)

    def IsOnGraph(self, graph):
        """
        Returns whether a datum is on a graph

        Parameters
        ----------
        graph : int
            Graph id

        Returns
        -------
        bool
            True if it is on the graph, False otherwise
        """
        return Oasys.THIS._connection.instanceMethod(self.__class__.__name__, self._handle, "IsOnGraph", graph)

    def Next(self):
        """
        Returns the next datum in the model

        Returns
        -------
        Datum
            Datum object (or None if there are no more datums in the model)
        """
        return Oasys.THIS._connection.instanceMethod(self.__class__.__name__, self._handle, "Next")

    def RemoveFromGraph(self, *graph):
        """
        Removes a datum from a graph

        Parameters
        ----------
        *graph : int
            Graph to remove the datum from. If undefined then the datum is removed from all graphs
            This argument can be repeated if required.
            Alternatively a single array argument containing the multiple values can be given

        Returns
        -------
        None
            No return value
        """
        return Oasys.THIS._connection.instanceMethod(self.__class__.__name__, self._handle, "RemoveFromGraph", graph)

